# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.
import copy

from keystoneauth1 import adapter

import mock

from openstack.tests.unit import base

from otcextensions.sdk.auto_scaling.v1 import policy

EXAMPLE = {
    'scaling_policy_id': 'fd7d63ce-8f5c-443e-b9a0-bef9386b23b3',
    'scaling_group_id': 'e5d27f5c-dd76-4a61-b4bc-a67c5686719a',
    'scaling_policy_name': 'schedule1',
    'scaling_policy_type': 'SCHEDULED',
    'scheduled_policy': {
        'launch_time': '2015-07-24T01:21Z'
    },
    'cool_down_time': 300,
    'scaling_policy_action': {
        'operation': 'REMOVE',
        'instance_number': 1
    },
    'policy_status': 'INSERVICE',
    'create_time': '2015-07-24T01:09:30Z'
}

EXAMPLE_LIST = {
    'limit': 20,
    'total_number': 1,
    'start_number': 0,
    'scaling_policies': [
        {
            'scaling_policy_id': 'fd7d63ce-8f5c-443e-b9a0-bef9386b23b3',
            'scaling_group_id': 'e5d27f5c-dd76-4a61-b4bc-a67c5686719a',
            'scaling_policy_name': 'schedule1',
            'scaling_policy_type': 'SCHEDULED',
            'scheduled_policy': {
                'launch_time': '2015-07-24T01:21Z'
            },
            'cool_down_time': 300,
            'scaling_policy_action': {
                'operation': 'REMOVE',
                'instance_number': 1
            },
            'policy_status': 'INSERVICE',
            'create_time': '2015-07-24T01:09:30Z'
        }
    ]
}


class TestPolicy(base.TestCase):

    def setUp(self):
        super(TestPolicy, self).setUp()
        self.sess = mock.Mock(spec=adapter.Adapter)
        self.sess.default_microversion = None
        self.sess.get = mock.Mock()
        self.sess.post = mock.Mock()
        self.sess.delete = mock.Mock()
        self.sess.put = mock.Mock()
        self.sess.retriable_status_codes = ()
        # self.sess.get_project_id = mock.Mock(return_value=PROJECT_ID)
        self.sot = policy.Policy()

    def test_basic(self):
        sot = policy.Policy()
        self.assertEqual('scaling_policy', sot.resource_key)
        self.assertEqual('scaling_policies', sot.resources_key)
        self.assertEqual('/scaling_policy', sot.base_path)
        self.assertTrue(sot.allow_list)
        self.assertTrue(sot.allow_create)
        self.assertTrue(sot.allow_fetch)
        self.assertTrue(sot.allow_commit)
        self.assertTrue(sot.allow_delete)

    def test_make_it(self):
        sot = policy.Policy.existing(**EXAMPLE)
        self.assertEqual(EXAMPLE['scaling_policy_id'], sot.id)
        self.assertEqual(EXAMPLE['scaling_policy_name'], sot.name)
        self.assertEqual(EXAMPLE['scaling_policy_type'], sot.type)
        self.assertEqual(EXAMPLE['scaling_group_id'], sot.scaling_group_id)
        self.assertEqual(EXAMPLE['cool_down_time'], sot.cool_down_time)
        self.assertEqual(EXAMPLE['policy_status'], sot.status)
        self.assertEqual(EXAMPLE['create_time'], sot.create_time)

    def test_get(self):
        sot = policy.Policy.existing(
            id=EXAMPLE['scaling_policy_id'])
        mock_response = mock.Mock()
        mock_response.status_code = 200
        mock_response.headers = {}
        mock_response.json.return_value = {
            'scaling_policy': EXAMPLE.copy()}

        self.sess.get.return_value = mock_response

        result = sot.fetch(self.sess)

        self.sess.get.assert_called_once_with(
            'scaling_policy/%s' %
            EXAMPLE['scaling_policy_id'],
            microversion=None,
            params={},
            skip_cache=False
        )

        self.assertEqual(sot, result)
        self.assertEqual(EXAMPLE['scaling_policy_id'], result.id)
        self.assertEqual(EXAMPLE['scaling_policy_name'], result.name)

    def test_create(self):
        mock_response = mock.Mock()
        mock_response.status_code = 200
        mock_response.headers = {}
        mock_response.json.return_value = {
            'scaling_policy': {
                'scaling_policy_id': EXAMPLE['scaling_policy_id']
            }}

        self.sess.post.return_value = mock_response

        sot = policy.Policy.new(**EXAMPLE)
        result = sot.create(self.sess, prepend_key=False)

        call_args = self.sess.post.call_args_list[0]

        expected_json = copy.deepcopy(EXAMPLE)

        self.assertEqual('/scaling_policy', call_args[0][0])
        self.assertDictEqual(expected_json, call_args[1]['json'])

        self.sess.post.assert_called_once()

        self.assertEqual(sot, result)
        self.assertEqual(EXAMPLE['scaling_policy_id'], result.id)
        self.assertEqual(EXAMPLE['scaling_policy_name'], result.name)

    def test_delete(self):
        mock_response = mock.Mock()
        mock_response.status_code = 200
        mock_response.headers = {}
        mock_response.json.return_value = {}

        self.sess.delete.return_value = mock_response

        sot = policy.Policy.existing(id=EXAMPLE['scaling_policy_id'])

        sot.delete(self.sess)

        self.sess.delete.assert_called_once()

    def test_update(self):
        mock_response = mock.Mock()
        mock_response.status_code = 200
        mock_response.headers = {}
        mock_response.json.return_value = {
            'scaling_policy': {
                'scaling_policy_id': EXAMPLE['scaling_policy_id']
            }
        }

        self.sess.put.return_value = mock_response

        sot = policy.Policy.existing(id=EXAMPLE['scaling_policy_id'])

        sot._update(**EXAMPLE)

        result = sot.commit(self.sess, prepend_key=False)

        call_args = self.sess.put.call_args_list[0]

        expected_json = copy.deepcopy(EXAMPLE)
        expected_json.pop('scaling_policy_id')

        self.assertEqual(
            'scaling_policy/%s' % EXAMPLE['scaling_policy_id'],
            call_args[0][0])
        self.assertDictEqual(expected_json, call_args[1]['json'])

        self.sess.put.assert_called_once()

        self.assertEqual(sot, result)
        self.assertEqual(EXAMPLE['scaling_policy_id'], result.id)
        self.assertEqual(EXAMPLE['scaling_policy_name'], result.name)

    def test_execute(self):
        mock_response = mock.Mock()
        mock_response.status_code = 200
        mock_response.headers = {}
        mock_response.json.return_value = {}

        self.sess.post.return_value = mock_response

        sot = policy.Policy.existing(id=EXAMPLE['scaling_policy_id'])

        sot.execute(self.sess)

        self.sess.post.assert_called_once_with(
            'scaling_policy/%s/action' % EXAMPLE['scaling_policy_id'],
            json={'action': 'execute'}
        )

    def test_pause(self):
        mock_response = mock.Mock()
        mock_response.status_code = 200
        mock_response.headers = {}
        mock_response.json.return_value = {}

        self.sess.post.return_value = mock_response

        sot = policy.Policy.existing(id=EXAMPLE['scaling_policy_id'])

        sot.pause(self.sess)

        self.sess.post.assert_called_once_with(
            'scaling_policy/%s/action' % EXAMPLE['scaling_policy_id'],
            json={'action': 'pause'}
        )

    def test_resume(self):
        mock_response = mock.Mock()
        mock_response.status_code = 200
        mock_response.headers = {}
        mock_response.json.return_value = {}

        self.sess.post.return_value = mock_response

        sot = policy.Policy.existing(id=EXAMPLE['scaling_policy_id'])

        sot.resume(self.sess)

        self.sess.post.assert_called_once_with(
            'scaling_policy/%s/action' % EXAMPLE['scaling_policy_id'],
            json={'action': 'resume'}
        )



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.
#
import uuid
from datetime import datetime

import mock

from openstackclient.tests.unit import utils

from otcextensions.sdk.identity.v3 import credential
from otcextensions.sdk.identity.v3 import custom_role
from otcextensions.tests.unit.osclient import test_base


def gen_data(data, columns):
    """Fill expected data tuple based on columns list
    """
    return tuple(getattr(data, attr, '') for attr in columns)


def gen_data_dict(data, columns):
    """Fill expected data tuple based on columns list
    """
    return tuple(data.get(attr, '') for attr in columns)


class TestIdentity(utils.TestCommand):
    def setUp(self):
        super(TestIdentity, self).setUp()

        self.app.client_manager.iam = mock.Mock()

        self.client = self.app.client_manager.iam


class FakeIdentityCredential(test_base.Fake):
    """Fake one or more identity credentials."""
    @classmethod
    def generate(cls):
        """Create a fake identity credential.

        :return:
            A FakeResource object, with id, name and so on
        """
        # Set default attributes.

        access = "id-" + uuid.uuid4().hex

        object_info = {
            "access": access,
            "created_at": datetime.now().strftime("%Y-%m-%d %H:%M:%S.%f"),
            "description": "my credential",
            "status": "active",
            "user_id": "user-id-" + uuid.uuid4().hex,
        }

        return credential.Credential(**object_info)


class FakeIdentityCustomRole(test_base.Fake):
    """Fake one or more identity custom roles."""
    @classmethod
    def generate(cls):
        """Create a fake identity custom role.

        :return:
            A FakeResource object, with id, name and so on
        """
        # Set default attributes.

        object_info = {
            "id": uuid.uuid4().hex,
            "name": "role" + uuid.uuid4().hex[:4],
            "description": "test",
            "domain_id": uuid.uuid4().hex,
            "catalog": "CUSTOMED",
            "display_name": "custom_" + uuid.uuid4().hex[:4],
            "type": "XA",
            "created_at": "1600181033358",
            "updated_at": "1600181033358",
        }
        return custom_role.CustomRole(**object_info)



#   Licensed under the Apache License, Version 2.0 (the "License"); you may
#   not use this file except in compliance with the License. You may obtain
#   a copy of the License at
#
#        http://www.apache.org/licenses/LICENSE-2.0
#
#   Unless required by applicable law or agreed to in writing, software
#   distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#   WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#   License for the specific language governing permissions and limitations
#   under the License.
import random
import uuid
import datetime

import mock

from otcextensions.sdk.sdrs.v1 import active_domains
from otcextensions.sdk.sdrs.v1 import job
from otcextensions.tests.unit.osclient import test_base

FAIL_REASON = "SdrsExtendReplicationPairNewTask-fail:SDRS.0011:Client " \
              "error : {\"badRequest\": {\"message\": \"Extending Replica" \
              "tion error: Check replication volume size or volume " \
              "status or volume quota fail, detail is f() takes exactly " \
              "1 argument (2 given)\", \"code\": 400}}"


def generate_entities():
    """Generate random list of sub_jobs"""
    jobs_list = []
    random_int = random.randint(1, 10)
    while random_int > 0:
        sub_job = {
            'id': uuid.uuid4().hex,
            'status': random.choice(['SUCCESS', 'FAIL', 'INIT']),
            'job_type': uuid.uuid4().hex,
            'begin_time': datetime.datetime.now(),
            'end_time': datetime.datetime.now(),
            'error_code': None,
            'fail_reason': None,
            'entities': {
                'server_group_id': uuid.uuid4().hex
            }
        }
        jobs_list.append(sub_job)
        random_int -= 1
    return jobs_list


class TestSDRS(test_base.TestCommand):

    def setUp(self):
        super(TestSDRS, self).setUp()

        self.app.client_manager.sdrs = mock.Mock()
        self.client = self.app.client_manager.sdrs


class FakeActiveDomain(test_base.Fake):
    """Fake one or more SDRS Active-active domains"""

    @classmethod
    def generate(cls):
        object_info = {
            'domains': [{
                'id': 'id-' + uuid.uuid4().hex,
                'name': 'name-' + uuid.uuid4().hex,
                'description': 'description-' + uuid.uuid4().hex,
                'sold_out': random.choice([True, False]),
                'local_replication_cluster': {
                    'availability_zone': random.choice(['eu-de-01',
                                                        'eu-de-02',
                                                        'eu-de-03'])
                },
                'remote_replication_cluster': {
                    'availability_zone': random.choice(['eu-de-01',
                                                        'eu-de-02',
                                                        'eu-de-03'])
                }
            }]
        }

        obj = active_domains.ActiveDomains.existing(**object_info)
        return obj


class FakeJob(test_base.Fake):
    """Fake one or more SDRS jobs"""

    @classmethod
    def generate(cls):
        object_info = {
            'job_id': uuid.uuid4().hex,
            'status': random.choice(['SUCCESS', 'FAIL', 'INIT']),
            'job_type': uuid.uuid4().hex,
            'begin_time': datetime.datetime.now(),
            'end_time': datetime.datetime.now()
        }

        entities = random.choice([True, False])

        if entities:
            object_info['entities'] = dict(sub_jobs=generate_entities())
        else:
            object_info['error_code'] = 'SDRS.001'
            object_info['fail_reason'] = FAIL_REASON

        obj = job.Job.existing(**object_info)
        return obj



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.
import mock

from keystoneauth1 import adapter
from openstack.tests.unit import base

from otcextensions.sdk.vpc.v1 import bandwidth

EXAMPLE = {
    'id': 'bandwidth-id',
    'name': 'bandwidth123',
    'size': 10,
    'share_type': 'WHOLE',
    'publicip_info': [
        {
            'publicip_id': 'publicip-id',
            'publicip_type': '5_bgp'
        }
    ],
    'bandwidth_type': 'share',
    'charge_mode': 'traffic',
    'billing_info': '',
    'enterprise_project_id': '0',
    'status': 'NORMAL',
    'created_at': '2020-04-21T07:58:02Z',
    'updated_at': '2020-04-21T07:58:02Z',
}


class TestBandwidth(base.TestCase):

    def setUp(self):
        super(TestBandwidth, self).setUp()
        self.sess = mock.Mock(spec=adapter.Adapter)
        self.sess.default_microversion = None
        self.sess.get = mock.Mock()
        self.sess.post = mock.Mock()
        self.sess.delete = mock.Mock()
        self.sess.put = mock.Mock()
        self.sess.retriable_status_codes = ()
        self.sot = bandwidth.Bandwidth()

    def test_basic(self):
        sot = bandwidth.Bandwidth()
        self.assertEqual('bandwidth', sot.resource_key)
        self.assertEqual('bandwidths', sot.resources_key)
        self.assertEqual('/%(version)s/%(project_id)s/bandwidths',
                         sot.base_path)
        self.assertTrue(sot.allow_list)
        self.assertTrue(sot.allow_create)
        self.assertTrue(sot.allow_fetch)
        self.assertTrue(sot.allow_commit)
        self.assertTrue(sot.allow_delete)

    def test_make_it(self):
        sot = bandwidth.Bandwidth.existing(**EXAMPLE)
        self.assertEqual(EXAMPLE['id'], sot.id)
        self.assertEqual(EXAMPLE['name'], sot.name)
        self.assertEqual(EXAMPLE['size'], sot.size)
        self.assertEqual(EXAMPLE['share_type'], sot.share_type)
        self.assertEqual(
            EXAMPLE['publicip_info'][0]['publicip_id'],
            sot.publicip_info[0]['publicip_id'])
        self.assertEqual(
            EXAMPLE['publicip_info'][0]['publicip_type'],
            sot.publicip_info[0]['publicip_type'])
        self.assertEqual(EXAMPLE['bandwidth_type'], sot.bandwidth_type)
        self.assertEqual(EXAMPLE['charge_mode'], sot.charge_mode)
        self.assertEqual(EXAMPLE['billing_info'], sot.billing_info)
        self.assertEqual(EXAMPLE['enterprise_project_id'],
                         sot.enterprise_project_id)
        self.assertEqual(EXAMPLE['status'], sot.status)
        self.assertEqual(EXAMPLE['created_at'], sot.created_at)
        self.assertEqual(EXAMPLE['updated_at'], sot.updated_at)



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.
#
"""DWS cluster v1 action implementations"""

import logging

from osc_lib import exceptions
from osc_lib import utils
from osc_lib.cli import format_columns
from osc_lib.command import command

from otcextensions.common import sdk_utils
from otcextensions.i18n import _

LOG = logging.getLogger(__name__)


_formatters = {
    'floating_ip': format_columns.DictColumn,
    'endpoints': format_columns.ListDictColumn,
    'public_endpoints': format_columns.ListDictColumn,
    'maintenance_window': format_columns.DictColumn,
    'parameter_group': format_columns.DictColumn,
    'private_ip': format_columns.ListColumn,
    'action_progress': format_columns.DictColumn,
    'public_domain': format_columns.ListColumn,
    'private_domain': format_columns.ListColumn,
    'nodes': format_columns.ListDictColumn,
    'plugins': format_columns.ListDictColumn,
}


def _get_columns(item):
    column_map = {}
    hidden = ['location', 'plugins']
    return sdk_utils.get_osc_show_columns_for_sdk_resource(
        item, column_map, hidden
    )


def set_attributes_for_print(obj):
    for data in obj:
        yield data


def translate_response(func):
    def new(self, *args, **kwargs):
        obj = func(self, *args, **kwargs)
        display_columns, columns = _get_columns(obj)
        data = utils.get_item_properties(obj, columns, formatters=_formatters)
        return (display_columns, data)

    new.__name__ = func.__name__
    new.__doc__ = func.__doc__
    return new


class ListClusters(command.Lister):
    _description = _('List DWS Clusters.')
    columns = (
        'ID',
        'Name',
        'Num Nodes',
        'Flavor',
        'Status',
        'Version',
        'Created At',
    )

    def get_parser(self, prog_name):
        parser = super(ListClusters, self).get_parser(prog_name)

        return parser

    def take_action(self, parsed_args):
        client = self.app.client_manager.dws
        data = client.clusters()

        return (
            self.columns,
            (utils.get_item_properties(s, self.columns) for s in data),
        )


class CreateCluster(command.ShowOne):
    _description = _('Create a new DWS cluster instance.')

    def get_parser(self, prog_name):
        parser = super(CreateCluster, self).get_parser(prog_name)
        parser.add_argument(
            'name',
            metavar='<name>',
            help=_('Cluster Name.'),
        )
        parser.add_argument(
            '--flavor',
            metavar='<flavor>',
            required=True,
            help=_('DWS Cluster Flavor (Node Type).'),
        )
        parser.add_argument(
            '--availability-zone',
            metavar='<availability_zone>',
            required=True,
            help=_('Availability Zone.'),
        )
        parser.add_argument(
            '--num-nodes',
            metavar='<num_nodes>',
            type=int,
            default=3,
            help=_(
                'Number of cluster Nodes. The value range is 3 to 256. '
                'For a hybrid data warehouse (standalone), the value is 1.'
            ),
        )
        parser.add_argument(
            '--num-cn',
            metavar='<num_cn>',
            type=int,
            help=_(
                'Number of deployed CNs. The value ranges from 2 to the '
                'number of cluster nodes minus 1. The maximum value is 20 '
                'and the default value is 3.'
            ),
        )

        network_group = parser.add_argument_group('Network Parameters')
        network_group.add_argument(
            '--router-id',
            metavar='<router_id>',
            required=True,
            help=_('Router ID.'),
        )
        network_group.add_argument(
            '--network-id',
            metavar='<network_id>',
            required=True,
            help=_('Network ID.'),
        )
        network_group.add_argument(
            '--security-group-id',
            metavar='<security_group_id>',
            required=True,
            help=_('Security group ID.'),
        )
        parser.add_argument(
            '--username',
            metavar='<username>',
            dest='user_name',
            required=True,
            help=_(
                'Administrator username for logging in to a '
                'GaussDB(DWS) cluster. The username must:\n'
                '- Consist of lowercase letters, digits, or underscores.\n'
                '- Start with a lowercase letter or an underscore.\n'
                '- Contain 1 to 63 characters.\n'
                '- Cannot be a keyword of the GaussDB(DWS) database.'
            ),
        )
        parser.add_argument(
            '--password',
            metavar='<password>',
            dest='user_pwd',
            required=True,
            help=_(
                'Administrator password for logging in to a '
                'GaussDB(DWS) cluster.'
            ),
        )
        parser.add_argument(
            '--port',
            metavar='<port>',
            default=8000,
            type=int,
            help=_(
                'Service port of a cluster. The value ranges from '
                '8000 to 30000. The default value is 8000.'
            ),
        )
        parser.add_argument(
            '--floating-ip',
            metavar='<floating_ip>',
            help=_(
                'Bind Floating Ip to a DWS Cluster.\n'
                'Possible values can be:\n'
                '- "auto" - To automatically assign Floating IP.\n'
                '- ID or IP of existing floating ip.'
            ),
        )
        parser.add_argument(
            '--enterprise-project-id',
            metavar='<enterprise_project_id>',
            help=_(
                'Enterprise project. The default '
                'enterprise project ID is 0.'
            ),
        )
        parser.add_argument(
            '--wait',
            action='store_true',
            help=('Wait for the Cluster status to be available.'),
        )
        parser.add_argument(
            '--timeout',
            metavar='<timeout>',
            type=int,
            default=1800,
            help=_('Timeout for the wait in seconds (Default 1800 seconds).'),
        )
        return parser

    @translate_response
    def take_action(self, parsed_args):
        client = self.app.client_manager.dws

        attrs = {}
        for arg in (
            'name',
            'router_id',
            'network_id',
            'security_group_id',
            'num_nodes',
            'user_name',
            'user_pwd',
            'flavor',
            'port',
            'availability_zone',
            'enterprise_project_id',
            'num_cn',
        ):
            val = getattr(parsed_args, arg)
            if val:
                attrs[arg] = val

        floating_ip = parsed_args.floating_ip

        if floating_ip and floating_ip.lower() == 'auto':
            attrs['public_ip'] = {
                'public_bind_type': 'auto_assign',
                'eip_id': ''
            }
        elif floating_ip:
            network_client = self.app.client_manager.network
            floating_ip_resp = network_client.find_ip(
                floating_ip, ignore_missing=False
            )
            attrs['public_ip'] = {
                'public_bind_type': 'bind_existing',
                'eip_id': floating_ip_resp.id,
            }
        cluster = client.create_cluster(**attrs)
        if parsed_args.wait:
            client.wait_for_cluster(cluster.id, wait=parsed_args.timeout)
        return client.get_cluster(cluster.id)


class ShowCluster(command.ShowOne):
    _description = _('Show details of a DWS cluster')

    def get_parser(self, prog_name):
        parser = super(ShowCluster, self).get_parser(prog_name)
        parser.add_argument(
            'cluster',
            metavar='<cluster>',
            help=_('Cluster name or ID.')
        )
        return parser

    @translate_response
    def take_action(self, parsed_args):
        client = self.app.client_manager.dws

        obj = client.find_cluster(parsed_args.cluster, ignore_missing=False)
        if obj.private_ip is None:
            obj = client.get_cluster(obj.id)
        return obj


class RestartCluster(command.Command):
    _description = _('Restart a DWS cluster')

    def get_parser(self, prog_name):
        parser = super(RestartCluster, self).get_parser(prog_name)
        parser.add_argument(
            'cluster',
            metavar='<cluster>',
            help=_('ID or Name of the DWS cluster to be restart.'),
        )
        parser.add_argument(
            '--wait',
            action='store_true',
            help=('Wait for the Cluster restart action to complete.'),
        )
        parser.add_argument(
            '--timeout',
            metavar='<timeout>',
            type=int,
            default=300,
            help=_('Timeout for the wait in seconds. (Default 300 seconds)'),
        )
        return parser

    def take_action(self, parsed_args):
        client = self.app.client_manager.dws
        cluster = client.find_cluster(
            parsed_args.cluster, ignore_missing=False
        )
        client.restart_cluster(cluster)
        if parsed_args.wait:
            client.wait_for_cluster(cluster.id, wait=parsed_args.timeout)


class ResetPassword(command.Command):
    _description = _('Reset the password of cluster administrator.')

    def get_parser(self, prog_name):
        parser = super(ResetPassword, self).get_parser(prog_name)
        parser.add_argument(
            'cluster',
            metavar='<cluster>',
            help=_('ID or Name of the DWS cluster to be restart.'),
        )
        parser.add_argument(
            '--password',
            metavar='<password>',
            required=True,
            help=_(
                'New password of the GaussDB(DWS) cluster administrator.\n'
                'A password must conform to the following rules:\n'
                '- Contains 8 to 32 characters.\n'
                '- Cannot be the same as the username or the username '
                'written in reverse order.\n'
                '- Contains at least three types of the following:\n'
                '- > Lowercase letters\n'
                '- > Uppercase letters\n'
                '- > Digits\n'
                '- > Special characters\n'
                '- Cannot be the same as previous passwords.\n'
                '- Cannot be a weak password.'
            ),
        )
        return parser

    def take_action(self, parsed_args):
        client = self.app.client_manager.dws
        cluster = client.find_cluster(
            parsed_args.cluster, ignore_missing=False
        )
        return client.reset_password(cluster, parsed_args.password)


class ScaleOutCluster(command.Command):
    _description = _('Scaling Out a Cluster with only Common Nodes.')

    def get_parser(self, prog_name):
        parser = super(ScaleOutCluster, self).get_parser(prog_name)
        parser.add_argument(
            'cluster',
            metavar='<cluster>',
            help=_('ID or Name of the DWS cluster to be extended.'),
        )
        parser.add_argument(
            '--add-nodes',
            metavar='<add_nodes>',
            type=int,
            required=True,
            help=_('Number of dws nodes to be scaled out.'),
        )
        parser.add_argument(
            '--wait',
            action='store_true',
            help=('Wait for the Cluster Scaling Task to complete.'),
        )
        parser.add_argument(
            '--timeout',
            metavar='<timeout>',
            type=int,
            default=1800,
            help=_('Timeout for the wait in seconds. (Default 1800 seconds)'),
        )
        return parser

    def take_action(self, parsed_args):
        client = self.app.client_manager.dws
        cluster = client.find_cluster(
            parsed_args.cluster, ignore_missing=False
        )
        client.scale_out_cluster(cluster, parsed_args.add_nodes)
        if parsed_args.wait:
            client.wait_for_cluster_scale_out(
                cluster.id, wait=parsed_args.timeout
            )


class DeleteCluster(command.Command):
    _description = _('Delete DWS Cluster(s)')

    def get_parser(self, prog_name):
        parser = super(DeleteCluster, self).get_parser(prog_name)
        parser.add_argument(
            'cluster',
            metavar='<cluster>',
            nargs='+',
            help=_('ID or Name of the DWS cluster(s) to be deleted.'),
        )
        parser.add_argument(
            '--keep-last-manual-snapshot',
            metavar='<keep_last_manual_snapshot>',
            type=int,
            default=0,
            help=_(
                'The number of latest manual snapshots that need '
                'to be retained for a cluster.'
            ),
        )
        return parser

    def take_action(self, parsed_args):
        client = self.app.client_manager.dws
        result = 0
        for name_or_id in parsed_args.cluster:
            try:
                cluster = client.find_cluster(name_or_id, ignore_missing=False)
                client.delete_cluster(
                    cluster.id, parsed_args.keep_last_manual_snapshot
                )
            except Exception as e:
                result += 1
                LOG.error(
                    _(
                        'Failed to delete cluster(s) with '
                        "ID or Name '%(cluster)s': %(e)s"
                    ),
                    {'cluster': name_or_id, 'e': e},
                )
        if result > 0:
            total = len(parsed_args.cluster)
            msg = _(
                '%(result)s of %(total)s Cluster(s) failed ' 'to delete.'
            ) % {'result': result, 'total': total}
            raise exceptions.CommandError(msg)



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.
from openstack import resource

from otcextensions.sdk import sdk_resource


class Metadata(sdk_resource.Resource):

    # Properties
    #: UUID
    #: *Type:str
    id = resource.Body('uuid', alternate_id=True)
    #: Name
    #: *Type:str
    name = resource.Body('name')
    #: Space UUID
    #: *Type:str
    space_uuid = resource.Body('spaceuuid')
    #: Create time
    #: *Type:str
    create_time = resource.Body('createAt')
    #: Update time
    #: *Type:str
    update_time = resource.Body('updateAt')


class Resource(sdk_resource.Resource):
    base_path = ''

    service_expectes_json_type = True

    # Properties
    #: Kind
    kind = resource.Body('kind')
    #: api version
    api_version = resource.Body('apiVersion', default='v1')
    #: metadata
    metadata = resource.Body('metadata', type=Metadata)



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.
from openstack.tests.unit import base

from otcextensions.common import format
from otcextensions.sdk.cts.v1 import trace

FAKE_ID = "68d5745e-6af2-40e4-945d-fe449be00148"
EXAMPLE = {
    "time": 1534748957969,
    "user": "{some crazy used struct}",
    "code": 302,
    "service_type": "IAM",
    "resource_type": "user",
    "resource_name": "username",
    "resource_id": "some_Resource_id",
    "source_ip": "8.8.8.8",
    "trace_name": "login",
    "trace_rating": "normal",
    "trace_type": "ConsoleAction",
    "record_time": 1534748957990,
    "trace_id": FAKE_ID
}


class TestTrace(base.TestCase):

    def test_basic(self):
        sot = trace.Trace()

        self.assertEqual('/%(tracker_name)s/trace', sot.base_path)

        self.assertTrue(sot.allow_list)

    def test_make_it(self):

        sot = trace.Trace(**EXAMPLE)
        self.assertEqual(EXAMPLE['trace_id'], sot.id)
        self.assertEqual(
            format.TimeTMsStr().deserialize(EXAMPLE['time']),
            sot.time),
        self.assertEqual(EXAMPLE['user'], sot.user)
        self.assertEqual(EXAMPLE['code'], sot.code)
        self.assertEqual(EXAMPLE['service_type'], sot.service_type)
        self.assertEqual(EXAMPLE['resource_type'], sot.resource_type)
        self.assertEqual(EXAMPLE['resource_name'], sot.resource_name)
        self.assertEqual(EXAMPLE['resource_id'], sot.resource_id)
        self.assertEqual(EXAMPLE['source_ip'], sot.source_ip)
        self.assertEqual(EXAMPLE['trace_name'], sot.name)
        self.assertEqual(EXAMPLE['trace_rating'], sot.level)
        self.assertEqual(EXAMPLE['trace_type'], sot.type)
        self.assertEqual(
            format.TimeTMsStr().deserialize(EXAMPLE['record_time']),
            sot.record_time)



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.
#
from openstack import exceptions
from openstack import resource


class Resource(resource.Resource):
    query_marker_key = "offset"

    _query_mapping = resource.QueryParameters(
        "marker", "limit", marker="offset"
    )

    @classmethod
    def list(
        cls,
        session,
        paginated=True,
        base_path=None,
        allow_unknown_params=False,
        **params
    ):
        if not cls.allow_list:
            raise exceptions.MethodNotSupported(cls, "list")
        session = cls._get_session(session)
        microversion = cls._get_microversion(session)

        if base_path is None:
            base_path = cls.base_path
        params = cls._query_mapping._validate(
            params,
            base_path=base_path,
            allow_unknown_params=allow_unknown_params,
        )

        query_params = cls._query_mapping._transpose(params, cls)
        uri = base_path % params

        limit = query_params.get("limit")

        # Track the total number of resources yielded so we can paginate
        # swift objects
        total_yielded = query_params.get("offset", 0)
        while uri:
            # Copy query_params due to weird mock unittest interactions
            response = session.get(
                uri,
                headers={"Accept": "application/json"},
                params=query_params.copy(),
                microversion=microversion,
            )
            exceptions.raise_from_response(response)
            data = response.json()
            # Discard any existing pagination keys
            query_params.pop("offset", None)
            query_params.pop("limit", None)

            if cls.resources_key:
                resources = data[cls.resources_key]
            else:
                resources = data

            if not isinstance(resources, list):
                resources = [resources]

            marker = None
            for raw_resource in resources:
                value = cls.existing(
                    microversion=microversion,
                    connection=session._get_connection(),
                    **raw_resource
                )
                marker = total_yielded + 1
                yield value
                total_yielded += 1

            if resources and paginated:
                uri, next_params = cls._get_next_link(
                    uri, response, data, marker, limit, total_yielded
                )
                query_params.update(next_params)
            else:
                return

    @classmethod
    def _get_next_link(cls, uri, response, data, marker, limit, total_yielded):
        # RDS service pagination. Returns query for the next page
        next_link = None
        params = {}
        if "total_count" in data and total_yielded < data["total_count"]:
            next_link = uri
            params["offset"] = marker
            params["limit"] = limit
        else:
            next_link = None
        query_params = cls._query_mapping._transpose(params, cls)
        return next_link, query_params



#   Licensed under the Apache License, Version 2.0 (the "License"); you may
#   not use this file except in compliance with the License. You may obtain
#   a copy of the License at
#
#        http://www.apache.org/licenses/LICENSE-2.0
#
#   Unless required by applicable law or agreed to in writing, software
#   distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#   WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#   License for the specific language governing permissions and limitations
#   under the License.
import datetime
import random
import uuid

import mock

from openstackclient.tests.unit import utils
from otcextensions.tests.unit.osclient import test_base

from otcextensions.sdk.kms.v1 import key
from otcextensions.sdk.kms.v1 import quota
# from otcextensions.sdk.kms.v1 import data_key


class TestKMS(utils.TestCommand):

    def setUp(self):
        super(TestKMS, self).setUp()

        self.app.client_manager.kms = mock.Mock()


class FakeCMK(test_base.Fake):
    """Fake one or more CMK"""

    @classmethod
    def generate(cls):
        object_info = {
            'creation_date': datetime.datetime(
                random.randint(2000, 2020),
                random.randint(1, 12),
                random.randint(1, 28)
            ),
            'scheduled_deletion_date': datetime.datetime(
                random.randint(2000, 2020),
                random.randint(1, 12),
                random.randint(1, 28)
            ),
            'key_alias': 'alias-' + uuid.uuid4().hex,
            'id': 'id-' + uuid.uuid4().hex,
            'domain_id': 'domain-' + uuid.uuid4().hex,
            'key_status': 'SOME STATUS',
            'key_description': 'description-' + uuid.uuid4().hex,
            'realm': 'realm-' + uuid.uuid4().hex,
            'key_type': 'type-' + uuid.uuid4().hex,
        }
        obj = key.Key.existing(**object_info)
        return obj


class FakeQuota(test_base.Fake):
    """Fake one or more Quota"""

    @classmethod
    def generate(cls):

        object_info = {
            'quota': random.randint(1, 65535),
            'used': random.randint(1, 65535),
            'type': random.choice(['CMK', 'grant_per_CMK']),
        }
        obj = quota.Quota.existing(**object_info)
        return obj



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.
#
"""Datastore v3 action implementations"""
from osc_lib import utils
from osc_lib.command import command

from otcextensions.i18n import _
from otcextensions.common import sdk_utils


DB_TYPE_CHOICES = ['mysql', 'postgresql', 'sqlserver']


def _get_columns(item):
    column_map = {}
    return sdk_utils.get_osc_show_columns_for_sdk_resource(item, column_map)


class ListDatastoreTypes(command.Lister):

    _description = _("List available datastores.")
    columns = ('Name', )

    def take_action(self, parsed_args):
        client = self.app.client_manager.rds

        data = client.datastore_types()

        return (self.columns, (utils.get_item_properties(
            s,
            self.columns,
        ) for s in data))


class ListDatastoreVersions(command.Lister):
    _description = _("Lists available versions for a datastore.")
    columns = ('ID', 'Name')

    def get_parser(self, prog_name):
        parser = super(ListDatastoreVersions, self).get_parser(prog_name)

        parser.add_argument(
            'database',
            metavar='{' + ','.join(DB_TYPE_CHOICES) + '}',
            type=lambda s: s.lower(),
            choices=DB_TYPE_CHOICES,
            help=_('Name of the datastore Engine.'),
        )
        return parser

    def take_action(self, parsed_args):
        client = self.app.client_manager.rds

        data = client.datastores(database_name=parsed_args.database)

        return (self.columns, (utils.get_item_properties(
            s,
            self.columns,
        ) for s in data))



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.
from keystoneauth1 import adapter

import mock

from openstack.tests.unit import base

from otcextensions.sdk.auto_scaling.v1 import activity

EXAMPLE_LIST = {
    'limit': 20,
    'scaling_activity_log': [
        {
            'id': 'a8924393-1024-4c24-8ac6-e4d481360884',
            'status': 'SUCCESS',
            'description': '{\'reason\':[{\'change_reason\':\'SCHEDULED\','
                           '\'old_value\':1,\'change_time\':\''
                           '2015-07-24T01:21:00Z\',\'new_value\':0}]}',
            'instance_value': 1,
            'desire_value': 0,
            'start_time': '2015-07-24T01:21:02Z',
            'end_time': '2015-07-24T01:23:31Z',
            'instance_added_list': 'as-config-TEO_XQF2JJSI',
            'instance_removed_list': 'as-config-TEO_XQF2JJSI',
            'instance_deleted_list': 'as-config-TEO_XQF2JJSI',
            'scaling_value': '0',
        },
        {
            'id': '423bbb2d-043c-4afe-8754-03e418a6ac42',
            'status': 'SUCCESS',
            'description': '{\'reason\':[{\'change_reason\':\'DIFF\','
                           '\'old_value\':0,\'change_time\':\''
                           '2015-07-23T15:11:52Z\',\'new_value\':1}]}',
            'instance_value': 0,
            'desire_value': 1,
            'start_time': '2015-07-23T15:11:52Z',
            'end_time': '2015-07-23T15:16:30Z',
            'instance_added_list': 'as-config-TEO_XQF2JJSI'
        }
    ],
    'total_number': 2,
    'start_number': 0
}


class TestActivity(base.TestCase):

    def setUp(self):
        super(TestActivity, self).setUp()
        self.sess = mock.Mock(spec=adapter.Adapter)
        self.sess.default_microversion = None
        self.sess.get = mock.Mock()
        self.sess._get_connection = mock.Mock(return_value=self.cloud)

        self.sot = activity.Activity()

    def test_basic(self):
        sot = activity.Activity()
        self.assertEqual(None, sot.resource_key)
        self.assertEqual('scaling_activity_log', sot.resources_key)
        self.assertEqual('/scaling_activity_log/%(scaling_group_id)s',
                         sot.base_path)
        self.assertTrue(sot.allow_list)
        self.assertFalse(sot.allow_create)
        self.assertFalse(sot.allow_commit)
        self.assertFalse(sot.allow_commit)
        self.assertFalse(sot.allow_delete)

    def test_make_it(self):
        obj = EXAMPLE_LIST['scaling_activity_log'][0]
        sot = activity.Activity.existing(**obj)
        self.assertEqual(obj['id'], sot.id)
        self.assertEqual(obj['status'], sot.status)
        self.assertEqual(obj['instance_value'], sot.instance_value)
        self.assertEqual(obj['scaling_value'], sot.scaling_value)
        self.assertEqual(obj['desire_value'], sot.desire_value)
        self.assertEqual(obj['start_time'], sot.start_time)
        self.assertEqual(obj['end_time'], sot.end_time)
        self.assertEqual(obj['instance_added_list'],
                         sot.instance_added_list)
        self.assertEqual(obj['instance_removed_list'],
                         sot.instance_removed_list)
        self.assertEqual(obj['instance_deleted_list'],
                         sot.instance_deleted_list)
        self.assertEqual(obj['description'], sot.description)



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.
from otcextensions.sdk import sdk_proxy
from otcextensions.sdk.mrs.v1 import cluster as _cluster
from otcextensions.sdk.mrs.v1 import datasource as _datasource
from otcextensions.sdk.mrs.v1 import jobbinary as _jobbinary
from otcextensions.sdk.mrs.v1 import job as _job


class Proxy(sdk_proxy.Proxy):
    skip_discovery = True

    def __init__(self, session, *args, **kwargs):
        super(Proxy, self).__init__(session=session, *args, **kwargs)
        self.additional_headers = {
            'Content-Type': 'application/json',
        }

    # ======== clusters ========
    def clusters(self, **query):
        """Retrieve a generator of clusters

        :param dict query: Optional query parameters to be sent to limit the
            resources being returned.
        :returns: A generator of cluster
            :class:`~otcextensions.sdk.mrs.v1.cluster.ClusterInfo` instances
        """
        return self._list(_cluster.ClusterInfo, **query)

    def get_cluster(self, cluster):
        """Get a cluster

        :param cluster: The value can be the ID or an instance of
            :class:`~otcextensions.sdk.cbr.v3.cluster.ClusterInfo`
        :returns: Cluster instance
        :rtype: :class:`~otcextensions.sdk.mrs.v1.cluster.ClusterInfo`
        """
        return self._get(_cluster.ClusterInfo, cluster)

    def find_cluster(self, name_or_id, ignore_missing=True):
        """Find a single Cluster by name or id

        :param name_or_id: The name or ID of a Cluster
        :param bool ignore_missing: When set to ``False``
            :class:`~openstack.exceptions.ResourceNotFound` will be raised
            when the group does not exist.
            When set to ``True``, no exception will be set when attempting
            to delete a nonexistent group.

        :returns: ``None``
        """
        return self._find(
            _cluster.ClusterInfo, name_or_id,
            ignore_missing=ignore_missing,
        )

    def update_cluster(self, cluster, **attrs):
        """Update Cluster attributes

        :param cluster: The id or an instance of
            :class:`~otcextensions.sdk.cbr.v1.cluster.ClusterInfo`
        :param dict attrs: attributes for update on
            :class:`~otcextensions.sdk.cbr.v1.cluster.ClusterInfo`

        :rtype: :class:`~otcextensions.sdk.cbr.v1.cluster.ClusterInfo`
        """
        return self._update(_cluster.ClusterInfo, cluster, **attrs)

    def delete_cluster(self, cluster, ignore_missing=True):
        """Delete (release) a cluster

        :param cluster: The value can be the ID of a cluster
             or a :class:`~otcextensions.sdk.mrs.v1.cluster.ClusterInfo`
             instance.
        :param bool ignore_missing: When set to ``False``
            :class:`~openstack.exceptions.ResourceNotFound` will be raised when
            the host does not exist.
            When set to ``True``, no exception will be set when attempting to
            delete a nonexistent host.

        :returns: Cluster been deleted
        :rtype: :class:`~otcextensions.sdk.mrs.v1.cluster.Cluster`
        """
        if isinstance(cluster, _cluster.ClusterInfo):
            cluster = cluster.id
        return self._delete(
            _cluster.Cluster,
            cluster,
            ignore_missing=ignore_missing)

    def hosts(self, **query):
        """Retrieve a generator of hosts

        :param dict query: Optional query parameters to be sent to limit the
            resources being returned.
        :returns: A generator of host
            :class:`~otcextensions.sdk.mrs.v1.cluster.Host` instances
        """
        return self._list(_cluster.Host, **query)

    # ======== datasources ========
    def datasources(self, **query):
        """Retrieve a generator of datasources

        :param dict query: Optional query parameters to be sent to limit the
            resources being returned.

            * `marker`:  pagination marker
            * `limit`: pagination limit

        :returns: A generator of datasoruce
            :class:`~otcextensions.sdk.mrs.v1.datasource.Datasource` instances
        """
        return self._list(_datasource.Datasource, paginated=True, **query)

    def create_datasource(self, **attrs):
        """Create (allocate) a new ds from attributes

        :param dict attrs: Keyword arguments which will be used to create
               a :class:`~otcextensions.sdk.mrs.v1.datasource.Datasource`,
               comprised of the properties on the Datasource class.
        :returns: The results of datasoruce creation
        :rtype: :class:`~otcextensions.sdk.mrs.v1.datasource.Datasource`
        """
        return self._create(_datasource.Datasource, prepend_key=False, **attrs)

    def get_datasource(self, datasource):
        """Get a datasource

        :param datasource: The value can be the ID or an instance of
            :class:`~otcextensions.sdk.mrs.v1.datasource.Datasource`
        :returns: Datasource instance
        :rtype: :class:`~otcextensions.sdk.mrs.v1.datasource.Datasource`
        """
        return self._get(_datasource.Datasource, datasource)

    def delete_datasource(self, datasource, ignore_missing=True):
        """Delete (release) a datasource

        :param datasource: The value can be the ID of a datasource
             or a :class:`~otcextensions.sdk.mrs.v1.datasource.Datasource` .
        :param bool ignore_missing: When set to ``False``
            :class:`~openstack.exceptions.ResourceNotFound` will be raised when
            the host does not exist.
            When set to ``True``, no exception will be set when attempting to
            delete a nonexistent host.

        :returns: datasource been deleted
        :rtype: :class:`~otcextensions.sdk.mrs.v1.datasource.Datasource`
        """
        return self._delete(_datasource.Datasource, datasource,
                            ignore_missing=ignore_missing)

    def find_datasource(self, name_or_id, ignore_missing=True):
        """Find a single datasource

        :param name_or_id: The name or ID of a datasource
        :param bool ignore_missing: When set to ``False``
            :class:`~openstack.exceptions.ResourceNotFound` will be raised
            when the host does not exist.
            When set to ``True``, no exception will be set when attempting
            to delete a nonexistent ds.

        :returns: ``None``
        """
        return self._find(_datasource.Datasource, name_or_id,
                          ignore_missing=ignore_missing)

    def update_datasource(self, datasource, **attrs):
        """Update ds attributes

        :param datasource: The id or an instance of
            :class:`~otcextensions.sdk.mrs.v1.datasource.Datasource`
        :param dict attrs: attributes for update on
            :class:`~otcextensions.sdk.mrs.v1.datasource.Datasource`

        :rtype: :class:`~otcextensions.sdk.mrs.v1.datasource.Datasource`
        """
        return self._update(_datasource.Datasource, datasource, **attrs)

    # ======== jobbinaries ========
    def jobbinaries(self, **query):
        """Retrieve a generator of jobbinaries

        :param dict query: Optional query parameters to be sent to limit the
            resources being returned.

            * `marker`:  pagination marker
            * `limit`: pagination limit

        :returns: A generator of jobbinary
            :class:`~otcextensions.sdk.mrs.v1.jobbinary.Jobbinary` instances
        """
        return self._list(_jobbinary.Jobbinary, paginated=True, **query)

    def create_jobbinary(self, **attrs):
        """Create a new jobbinary

        :param dict attrs: Keyword arguments which will be used to create
               a :class:`~otcextensions.sdk.mrs.v1.jobbinary.Jobbinary`,
               comprised of the properties on the Jobbinary class.
        :returns: The results of datasoruce creation
        :rtype: :class:`~otcextensions.sdk.mrs.v1.jobbinary.Jobbinary`
        """
        return self._create(_jobbinary.Jobbinary, prepend_key=False, **attrs)

    def get_jobbinary(self, jobbinary):
        """Get a jobbinary

        :param jobbinary: The value can be the ID or an instance of
            :class:`~otcextensions.sdk.mrs.v1.jobbinary.Jobbinary`
        :returns: Jobbinary instance
        :rtype: :class:`~otcextensions.sdk.mrs.v1.jobbinary.Jobbinary`
        """
        return self._get(_jobbinary.Jobbinary, jobbinary)

    def delete_jobbinary(self, jobbinary, ignore_missing=True):
        """Delete (release) a jobbinary

        :param jobbinary: The value can be the ID of a jobbinary
             or a :class:`~otcextensions.sdk.mrs.v1.jobbinary.Jobbinary` .
        :param bool ignore_missing: When set to ``False``
            :class:`~openstack.exceptions.ResourceNotFound` will be raised when
            the host does not exist.
            When set to ``True``, no exception will be set when attempting to
            delete a nonexistent host.

        :rtype: :class:`~otcextensions.sdk.mrs.v1.jobbinary.Jobbinary`
        """
        return self._delete(_jobbinary.Jobbinary, jobbinary,
                            ignore_missing=ignore_missing)

    def find_jobbinary(self, name_or_id, ignore_missing=True):
        """Find a single jobbinary

        :param name_or_id: The name or ID of a jobbinary
        :param bool ignore_missing: When set to ``False``
            :class:`~openstack.exceptions.ResourceNotFound` will be raised
            when the host does not exist.
            When set to ``True``, no exception will be set when attempting
            to delete a nonexistent ds.

        :returns: ``None``
        """
        return self._find(_jobbinary.Jobbinary, name_or_id,
                          ignore_missing=ignore_missing)

    def update_jobbinary(self, jobbinary, **attrs):
        """Update jobbinary attributes

        :param jobbinary: The id or an instance of
            :class:`~otcextensions.sdk.mrs.v1.jobbinary.Jobbinary`
        :param dict attrs: attributes for update on
            :class:`~otcextensions.sdk.mrs.v1.jobbinary.Jobbinary`

        :rtype: :class:`~otcextensions.sdk.mrs.v1.jobbinary.Jobbinary`
        """
        return self._update(_jobbinary.Jobbinary, jobbinary, **attrs)

    # ======== jobs ========
    def jobs(self, **query):
        """Retrieve a generator of jobs

        :param dict query: Optional query parameters to be sent to limit the
            resources being returned.

            * `marker`:  pagination marker
            * `limit`: pagination limit

        :returns: A generator of jobs
            :class:`~otcextensions.sdk.mrs.v1.job.Job` instances
        """
        return self._list(_job.Job, paginated=True, **query)

    def create_job(self, **attrs):
        """Create (allocate) a new job from attributes

        :param dict attrs: Keyword arguments which will be used to create
               a :class:`~otcextensions.sdk.mrs.v1.job.Job`,
               comprised of the properties on the Job class.
        :returns: The results of job creation
        :rtype: :class:`~otcextensions.sdk.mrs.v1.job.Job`
        """
        return self._create(_job.Job, prepend_key=False, **attrs)

    def get_job(self, job):
        """Get a job

        :param job: The value can be the ID or an instance of
            :class:`~otcextensions.sdk.mrs.v1.job.Job`
        :returns: Job instance
        :rtype: :class:`~otcextensions.sdk.mrs.v1.job.Job`
        """
        return self._get(_job.Job, job)

    def delete_job(self, job, ignore_missing=True):
        """Delete (release) a job

        :param job: The value can be the ID of a job
             or a :class:`~otcextensions.sdk.mrs.v1.job.Job` .
        :param bool ignore_missing: When set to ``False``
            :class:`~openstack.exceptions.ResourceNotFound` will be raised when
            the host does not exist.
            When set to ``True``, no exception will be set when attempting to
            delete a nonexistent host.

        :returns: Job been deleted
        :rtype: :class:`~otcextensions.sdk.mrs.v1.job.Job`
        """
        return self._delete(_job.Job, job,
                            ignore_missing=ignore_missing)

    def find_job(self, name_or_id, ignore_missing=True):
        """Find a single job

        :param name_or_id: The name or ID of a job
        :param bool ignore_missing: When set to ``False``
            :class:`~openstack.exceptions.ResourceNotFound` will be raised
            when the host does not exist.
            When set to ``True``, no exception will be set when attempting
            to delete a nonexistent job.

        :returns: ``None``
        """
        return self._find(_job.Job, name_or_id,
                          ignore_missing=ignore_missing)

    def update_job(self, job, **attrs):
        """Update job attributes

        :param job: The id or an instance of
            :class:`~otcextensions.sdk.mrs.v1.job.Job`
        :param dict attrs: attributes for update on
            :class:`~otcextensions.sdk.mrs.v1.job.Job`

        :rtype: :class:`~otcextensions.sdk.mrs.v1.job.Job`
        """
        return self._update(_job.Job, job, **attrs)



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

from openstack.tests.unit import base

from otcextensions.sdk.mrs.v1 import cluster

EXAMPLE = {
    "id": "063d1d47-ae91-4a48-840c-b3cfe4efbcf0",
    "name": "a78e161c-d14f-4b68-8c2d-0219920ce844_node_core_IQhiC",
    "ip": "192.168.0.169",
    "status": "ACTIVE",
    "flavor": "c2.2xlarge.linux.mrs",
    "type": "Core",
    "mem": "16384",
    "cpu": "8",
    "root_volume_size": "40",
    "data_volume_type": "SATA",
    "data_volume_size": 100,
    "data_volume_count": 1
}


class TestHost(base.TestCase):

    def test_basic(self):
        sot = cluster.Host()
        self.assertEqual('hosts', sot.resources_key)
        path = '/clusters/%(cluster_id)s/hosts'
        self.assertEqual(path, sot.base_path)
        self.assertTrue(sot.allow_list)
        self.assertFalse(sot.allow_create)
        self.assertTrue(sot.allow_get)
        self.assertFalse(sot.allow_commit)
        self.assertFalse(sot.allow_delete)
        self.assertFalse(sot.allow_update)

    def test_make_it(self):
        sot = cluster.Host(**EXAMPLE)
        self.assertEqual(EXAMPLE['id'], sot.id)
        self.assertEqual(EXAMPLE['ip'], sot.ip)
        self.assertEqual(EXAMPLE['mem'], sot.mem)
        self.assertEqual(EXAMPLE['name'], sot.name)
        self.assertEqual(EXAMPLE['status'], sot.status)
        self.assertEqual(EXAMPLE['type'], sot.type)
        self.assertEqual(EXAMPLE['flavor'], sot.flavor)
        self.assertEqual(EXAMPLE['cpu'], sot.cpu)
        self.assertEqual(EXAMPLE['data_volume_size'], sot.data_volume_size)



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.
from openstack import resource

from otcextensions.sdk.auto_scaling.v1 import _base


class Group(_base.Resource):
    resource_key = 'scaling_group'
    resources_key = 'scaling_groups'
    base_path = '/scaling_group'
    query_marker_key = 'start_number'
    # service = auto_scaling_service.AutoScalingService()

    # capabilities
    allow_create = True
    allow_list = True
    allow_fetch = True
    allow_delete = True
    allow_commit = True

    _query_mapping = resource.QueryParameters(
        'id', 'name', 'limit', 'marker',
        'scaling_configuration_id',
        name='scaling_group_name',
        status='scaling_group_status',
        marker=query_marker_key
    )

    #: Properties
    #: AutoScaling group ID
    id = resource.Body('scaling_group_id', alternate_id=True)
    #: AutoScaling group name
    name = resource.Body('scaling_group_name')
    #: AutoScaling group status,
    #: valid valus includes: ``INSERVICE``, ``PAUSED``, ``ERROR``
    status = resource.Body('scaling_group_status')
    #: AutoScaling group scaling status, *Type: bool*
    is_scaling = resource.Body('is_scaling', type=bool)
    #: AutoScaling group detail
    detail = resource.Body('detail')
    #: VPC id - (Router Id)
    router_id = resource.Body('vpc_id')
    #: network id list - (Subnet)
    networks = resource.Body('networks', type=list)
    #: security group id list
    security_groups = resource.Body('security_groups', type=list)
    #: Auto Scaling Config ID reference, used for creating instance
    scaling_configuration_id = resource.Body('scaling_configuration_id')
    #: Auto Scaling Config name
    scaling_configuration_name = resource.Body('scaling_configuration_name')
    #: Current alive instance number
    current_instance_number = resource.Body('current_instance_number')
    #: Desire alive instance number
    desire_instance_number = resource.Body('desire_instance_number')
    #: min alive instance number
    min_instance_number = resource.Body('min_instance_number')
    #: max alive instance number
    max_instance_number = resource.Body('max_instance_number')
    #: CoolDown time, only work with `ALARM` policy.
    #: default is 900, valid range is 0-86400
    cool_down_time = resource.Body('cool_down_time')
    #: load balancer listener id reference
    lb_listener_id = resource.Body('lb_listener_id')
    #: list of enhanced load balancers
    lbaas_listeners = resource.Body('lbaas_listeners')
    #: Health periodic audit method, Valid values include: ``ELB_AUDIT``,
    #: ``NOVA_AUDIT``, ELB_AUDIT and lb_listener_id are used in pairs.
    health_periodic_audit_method = resource.Body(
        'health_periodic_audit_method')
    #: Health periodic audit time, valid values include: ``5``, ``15``,
    #: ``60``, ``180``, default is ``5`` minutes
    health_periodic_audit_time = resource.Body('health_periodic_audit_time')
    #: Grace period for instance health check, valid if audit method is
    #: ``ELB_AUDIT``, value range is 0-86400, default value is 600
    health_periodic_audit_grace_period = \
        resource.Body('health_periodic_audit_grace_period')
    #: Instance terminate policy, valid values include:
    #: ``OLD_CONFIG_OLD_INSTANCE`` (default), ``OLD_CONFIG_NEW_INSTANCE``,
    #: ``OLD_INSTANCE``, ``NEW_INSTANCE``
    instance_terminate_policy = resource.Body('instance_terminate_policy')
    #: Notification methods, ``EMAIL``
    notifications = resource.Body('notifications')
    #: Should delete public ip when terminate instance, default ``false``
    delete_publicip = resource.Body('delete_publicip', type=bool)
    #: Should delete data disks when deleting the ECS, default ``false``
    delete_volume = resource.Body('delete_volume', type=bool)
    #: Availability zones
    availability_zones = resource.Body('available_zones')
    #: Create time of the group
    create_time = resource.Body('create_time')
    #: The priority policy used to select target AZs, valid values include:
    #: ``EQUILIBRIUM_DISTRIBUTE`` (default), ``PICK_FIRST``
    multi_az_priority_policy = resource.Body('multi_az_priority_policy')

    def resume(self, session):
        '''resume group'''
        body = {'action': 'resume'}
        self._action(session, body)

    def pause(self, session):
        '''pause group'''
        body = {'action': 'pause'}
        self._action(session, body)

    def delete(self, session, error_message=None, force_delete=False):
        """Delete the remote resource based on this instance.

        This function overrides default Resource.delete to enable params

        :param session: The session to use for making this request.
        :type session: :class:`~keystoneauth1.adapter.Adapter`

        :return: This :class:`Group` instance.
        """

        params = {}
        if force_delete:
            params["force_delete"] = "yes"
        request = self._prepare_request(params=params)

        session = self._get_session(session)
        microversion = self._get_microversion(session)

        response = session.delete(request.url, headers=request.headers,
                                  microversion=microversion)

        kwargs = {}
        if error_message:
            kwargs['error_message'] = error_message

        self._translate_response(response, has_body=False, **kwargs)
        return self



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.
#
import uuid
from datetime import datetime

import mock
from openstackclient.tests.unit import utils
from osc_lib import utils as _osc_lib_utils

from otcextensions.sdk.vpcep.v1 import connection
from otcextensions.sdk.vpcep.v1 import endpoint
from otcextensions.sdk.vpcep.v1 import quota
from otcextensions.sdk.vpcep.v1 import service
from otcextensions.sdk.vpcep.v1 import whitelist
from otcextensions.tests.unit.osclient import test_base


def gen_data(obj, columns, formatters=None):
    """Fill expected data tuple based on columns list"""
    return _osc_lib_utils.get_item_properties(
        obj, columns, formatters=formatters
    )


def gen_data_dict(data, columns):
    """Fill expected data tuple based on columns list"""
    return tuple(data.get(attr, '') for attr in columns)


class TestVpcep(utils.TestCommand):
    def setUp(self):
        super(TestVpcep, self).setUp()

        self.app.client_manager.vpcep = mock.Mock()

        self.client = self.app.client_manager.vpcep


class FakeService(test_base.Fake):
    """Fake one or more endpoint service(s)."""

    @classmethod
    def generate(cls):
        """Create a fake endpoint service.
        :return:
            A FakeResource object, with id, name and so on
        """
        # Set default attributes.
        object_info = {
            'id': 'id-' + uuid.uuid4().hex,
            'port_id': 'port-id-' + uuid.uuid4().hex,
            'vpc_id': 'vpc-id-' + uuid.uuid4().hex,
            'pool_id': 'pool-id-' + uuid.uuid4().hex,
            'status': 'available',
            'approval_enabled': False,
            'service_name': 'vpcep-service-' + uuid.uuid4().hex,
            'service_type': 'interface',
            'server_type': 'VM',
            'project_id': 'project-id-' + uuid.uuid4().hex,
            'created_at': datetime.now().strftime('%Y-%m-%d %H:%M:%S.%f'),
            'ports': [
                {'client_port': 8080, 'server_port': 90, 'protocol': 'TCP'},
                {'client_port': 8081, 'server_port': 80, 'protocol': 'TCP'},
            ],
        }

        return service.Service(**object_info)


class FakeEndpoint(test_base.Fake):
    """Fake one or more Endpoint(s)."""

    @classmethod
    def generate(cls):
        """Create a fake endpoint.
        :return:
            A FakeResource object, with id, status and so on
        """
        # Set default attributes.
        object_info = {
            'id': 'id-' + uuid.uuid4().hex,
            'service_type': 'interface',
            'marker_id': 322312312312,
            'status': 'creating',
            'vpc_id': 'vpc-id-' + uuid.uuid4().hex,
            'enable_dns': False,
            'is_enabled': True,
            'endpoint_service_name': 'vpcep-' + uuid.uuid4().hex,
            'endpoint_service_id': 'ep-service-id-' + uuid.uuid4().hex,
            'project_id': 'project-id-' + uuid.uuid4().hex,
            'whitelist': ['127.0.0.1'],
            'enable_whitelist': True,
            'created_at': datetime.now().strftime('%Y-%m-%d %H:%M:%S.%f'),
            'updated_at': datetime.now().strftime('%Y-%m-%d %H:%M:%S.%f'),
            'tags': [{'key': 'test1', 'value': 'test1'}],
        }

        return endpoint.Endpoint(**object_info)


class FakeWhitelist(test_base.Fake):
    """Fake one or more Whitelist."""

    @classmethod
    def generate(cls):
        """Create a fake service whitelist.
        :return:
            A FakeResource object, with id, status and so on
        """
        # Set default attributes.
        object_info = {
            'id': 'id-' + uuid.uuid4().hex,
            'permission': '*',
            'created_at': datetime.now().strftime('%Y-%m-%d %H:%M:%S.%f'),
        }
        return whitelist.Whitelist(**object_info)


class FakeConnection(test_base.Fake):
    """Fake one or more Connection(s)."""

    @classmethod
    def generate(cls):
        """Create a fake service connection.
        :return:
            A FakeResource object, with id, status and so on
        """
        # Set default attributes.
        object_info = {
            'id': 'id-' + uuid.uuid4().hex,
            'status': 'accepted',
            'marker_id': 16777510,
            'domain_id': '5fc973eea581490997e82ea11a1df31f',
            'created_at': '2018-09-17T11:10:11Z',
            'updated_at': '2018-09-17T11:10:12Z',
        }
        return connection.Connection(**object_info)


class FakeQuota(test_base.Fake):
    """Fake one or more resource Quota(s)."""

    @classmethod
    def generate(cls):
        """Create a fake resource quota.
        :return:
            A FakeResource object, with id, status and so on
        """
        # Set default attributes.
        object_info = {'type': 'endpoint', 'quota': 50, 'used': 4}
        return quota.Quota(**object_info)



#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

import json
import uuid

from openstackclient.tests.functional import base
from tempest.lib import exceptions


class TestRdsFlavor(base.TestCase):
    """Functional tests for RDS Flavor. """

    NAME = uuid.uuid4().hex
    OTHER_NAME = uuid.uuid4().hex

    def test_flavor_list(self):
        datastores = json.loads(self.openstack(
            'rds datastore type list -f json'
        ))
        for datastore in datastores:
            datastore = datastore['Name']
            json_output = json.loads(self.openstack(
                'rds datastore version list ' + datastore + ' -f json '
            ))

            for ds_ver in json_output:
                json_output = json.loads(self.openstack(
                    'rds flavor list {ds} {ver} -f json'.format(
                        ds=datastore,
                        ver=ds_ver['Name'])
                ))

                self.assertIsNotNone(json_output)
                if len(json_output) > 0:
                    self.assertEqual(
                        ['name', 'instance_mode', 'vcpus', 'ram'],
                        list(json_output[0].keys())
                    )

    def test_invalid_datastore_flavor_list(self):
        self.assertRaises(
            exceptions.CommandFailed,
            self.openstack,
            'rds flavor list'
        )

        self.assertRaises(
            exceptions.CommandFailed,
            self.openstack,
            'rds flavor list invalid_ds 5.6'
        )

        datastores = json.loads(self.openstack(
            'rds datastore type list -f json'
        ))
        for datastore in datastores:
            datastore = datastore['Name']
            self.assertRaises(
                exceptions.CommandFailed,
                self.openstack,
                'rds flavor list {ds} {ver}'.format(
                    ds=datastore,
                    ver=0.0)
            )



#   Licensed under the Apache License, Version 2.0 (the "License"); you may
#   not use this file except in compliance with the License. You may obtain
#   a copy of the License at
#
#        http://www.apache.org/licenses/LICENSE-2.0
#
#   Unless required by applicable law or agreed to in writing, software
#   distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#   WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#   License for the specific language governing permissions and limitations
#   under the License.
#
import mock

from openstackclient.tests.unit import utils

from otcextensions.common import sdk_utils
from otcextensions.osclient.load_balancer.v1 import listener
from otcextensions.tests.unit.osclient.load_balancer.v1 import fakes


class TestListListener(fakes.TestLoadBalancer):

    _objects = fakes.FakeListener.create_multiple(3)

    columns = (
        'id', 'default_pool_id', 'name', 'project_id',
        'protocol', 'protocol_port', 'admin_state_up')

    data = []

    for s in _objects:
        data.append((
            s.id,
            s.default_pool_id,
            s.name,
            s.project_id,
            s.protocol,
            s.protocol_port,
            # sdk_utils.ListOfIdsColumn(s.load_balancer_ids),
            s.is_admin_state_up,
        ))

    def setUp(self):
        super(TestListListener, self).setUp()

        self.cmd = listener.ListListener(self.app, None)

        self.client.listeners = mock.Mock()

    def test_list_default(self):
        arglist = [
        ]

        verifylist = [
        ]
        # Verify cm is triggereg with default parameters
        parsed_args = self.check_parser(self.cmd, arglist, verifylist)

        # Set the response
        self.client.listeners.side_effect = [
            self._objects
        ]

        # Trigger the action
        columns, data = self.cmd.take_action(parsed_args)

        self.client.listeners.assert_called_once_with()

        self.assertEqual(self.columns, columns)
        self.assertListItemEqual(self.data, list(data))

    def test_list_filters(self):
        arglist = [
            '--protocol', 'TCP',
            '--protocol_port', '12',
            '--name', 'some_name',
            '--load_balancer', 'lb'
        ]

        verifylist = [
            ('protocol', 'TCP'),
            ('protocol_port', 12),
            ('name', 'some_name'),
            ('load_balancer', 'lb')
        ]
        # Verify cm is triggereg with default parameters
        parsed_args = self.check_parser(self.cmd, arglist, verifylist)

        # Set the response
        self.client.listeners.side_effect = [
            self._objects
        ]

        # Trigger the action
        columns, data = self.cmd.take_action(parsed_args)

        self.client.listeners.assert_called_once_with(
            protocol='TCP',
            protocol_port=12,
            name='some_name',
            load_balancer_id='lb'
        )

        self.assertEqual(self.columns, columns)
        self.assertListItemEqual(self.data, list(data))

    def test_list_filters_exceptions_proto(self):
        arglist = [
            '--protocol', 'SMTP',
            '--protocol_port', '12'
        ]

        verifylist = [
            ('protocol', 'SMTP'),
            ('protocol_port', 12)
        ]

        self.assertRaises(
            utils.ParserException,
            self.check_parser, self.cmd, arglist, verifylist)

    def test_list_filters_exceptions_port(self):
        arglist = [
            '--protocol_port', '12x'
        ]

        verifylist = [
            ('protocol_port', 12)
        ]

        # Ensure exception is raised
        self.assertRaises(
            utils.ParserException,
            self.check_parser, self.cmd, arglist, verifylist)


class TestShowListener(fakes.TestLoadBalancer):

    _object = fakes.FakeListener.create_one()

    columns = (
        'admin_state_up', 'connection_limit', 'default_pool_id',
        'description', 'id', 'loadbalancers', 'name', 'protocol',
        'protocol_port')

    data = (
        _object.is_admin_state_up,
        _object.connection_limit,
        _object.default_pool_id,
        _object.description,
        _object.id,
        sdk_utils.ListOfIdsColumnBR(_object.load_balancer_ids),
        _object.name,
        _object.protocol,
        _object.protocol_port,
    )

    def setUp(self):
        super(TestShowListener, self).setUp()

        self.cmd = listener.ShowListener(self.app, None)

        self.client.find_listener = mock.Mock()

    def test_show_default(self):
        arglist = [
            'lb'
        ]

        verifylist = [
            ('listener', 'lb')
        ]
        # Verify cm is triggereg with default parameters
        parsed_args = self.check_parser(self.cmd, arglist, verifylist)

        # Set the response
        self.client.find_listener.side_effect = [
            self._object
        ]

        # Trigger the action
        columns, data = self.cmd.take_action(parsed_args)

        self.client.find_listener.assert_called_once_with(
            name_or_id='lb',
            ignore_missing=False
        )

        self.assertEqual(self.columns, columns)
        self.assertItemEqual(self.data, data)


class TestCreateListener(fakes.TestLoadBalancer):

    _object = fakes.FakeListener.create_one()

    columns = (
        'admin_state_up', 'connection_limit', 'default_pool_id',
        'description', 'id', 'loadbalancers', 'name', 'protocol',
        'protocol_port')

    data = (
        _object.is_admin_state_up,
        _object.connection_limit,
        _object.default_pool_id,
        _object.description,
        _object.id,
        sdk_utils.ListOfIdsColumnBR(_object.load_balancer_ids),
        _object.name,
        _object.protocol,
        _object.protocol_port,
    )

    def setUp(self):
        super(TestCreateListener, self).setUp()

        self.cmd = listener.CreateListener(self.app, None)

        self.client.create_listener = mock.Mock()

    def test_create_default(self):
        arglist = [
            'lb',
            '--protocol', 'tcp',
            '--protocol_port', '134',
            '--disable',
            '--connection_limit', '-1',
            '--default_pool', 'pool',
            '--default_tls_container_ref', 'default_tls_container_ref',
            '--description', 'description',
            '--name', 'name'
        ]

        verifylist = [
            ('protocol', 'TCP'),
            ('protocol_port', 134),
            ('load_balancer', 'lb'),
            ('disable', True),
            ('connection_limit', -1),
            ('default_pool', 'pool'),
            ('default_tls_container_ref', 'default_tls_container_ref'),
            ('description', 'description'),
            ('name', 'name')
        ]

        # Verify cm is triggereg with default parameters
        parsed_args = self.check_parser(self.cmd, arglist, verifylist)

        # Set the response
        self.client.create_listener.side_effect = [
            self._object
        ]

        # Trigger the action
        columns, data = self.cmd.take_action(parsed_args)

        self.client.create_listener.assert_called_once_with(
            connection_limit=-1,
            default_pool_id='pool',
            default_tls_container_ref='default_tls_container_ref',
            description='description',
            is_admin_state_up=False,
            load_balancer_id='lb',
            name='name',
            protocol='TCP',
            protocol_port=134
        )

        self.assertEqual(self.columns, columns)
        self.assertItemEqual(self.data, data)


class TestUpdateListener(fakes.TestLoadBalancer):

    _object = fakes.FakeListener.create_one()

    columns = (
        'ID', 'Name', 'description',
        'is_admin_state_up', 'protocol', 'protocol_port',
        'load_balancer_ids', 'default_pool_id',
        'connection_limit')

    data = (
        _object.id,
        _object.name,
        _object.description,
        _object.is_admin_state_up,
        _object.protocol,
        _object.protocol_port,
        sdk_utils.ListOfIdsColumn(_object.load_balancer_ids),
        _object.default_pool_id,
        _object.connection_limit,
    )

    def setUp(self):
        super(TestUpdateListener, self).setUp()

        self.cmd = listener.SetListener(self.app, None)

        self.client.update_listener = mock.Mock()

    def test_update_default(self):
        arglist = [
            'lsnr',
            '--disable',
            '--connection_limit', '-1',
            '--default_pool', 'pool',
            '--default_tls_container_ref', 'default_tls_container_ref',
            '--description', 'description',
            '--name', 'name'
        ]

        verifylist = [
            ('listener', 'lsnr'),
            ('disable', True),
            ('connection_limit', -1),
            ('default_pool', 'pool'),
            ('default_tls_container_ref', 'default_tls_container_ref'),
            ('description', 'description'),
            ('name', 'name')
        ]

        # Verify cm is triggereg with default parameters
        parsed_args = self.check_parser(self.cmd, arglist, verifylist)

        # Set the response
        self.client.update_listener.side_effect = [
            self._object
        ]

        # Trigger the action
        columns, data = self.cmd.take_action(parsed_args)

        self.client.update_listener.assert_called_once_with(
            is_admin_state_up=False,
            listener='lsnr',
            connection_limit=-1,
            default_pool_id='pool',
            default_tls_container_ref='default_tls_container_ref',
            description='description',
            name='name',
        )


class TestDeleteListener(fakes.TestLoadBalancer):

    _object = fakes.FakeListener.create_one()

    def setUp(self):
        super(TestDeleteListener, self).setUp()

        self.cmd = listener.DeleteListener(self.app, None)

        self.client.delete_listener = mock.Mock()
        self.client.find_listener = mock.Mock()

    def test_delete_default(self):
        arglist = [
            'lsnr',
        ]

        verifylist = [
            ('listener', ['lsnr']),
        ]

        # Verify cm is triggereg with default parameters
        parsed_args = self.check_parser(self.cmd, arglist, verifylist)

        # Set the response
        self.client.delete_listener.side_effect = [
            {}
        ]
        self.client.find_listener.side_effect = [
            self._object
        ]

        # Trigger the action
        self.cmd.take_action(parsed_args)

        self.client.delete_listener.assert_called_once_with(
            listener=self._object.id,
            ignore_missing=False
        )



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.
import openstack
import uuid

from otcextensions.tests.functional.sdk.dns import TestDns

_logger = openstack._log.setup_logging('openstack')


class TestRecordsets(TestDns):
    uuid_v4 = uuid.uuid4().hex[:8]
    zone_alias = uuid_v4 + 'dns.sdk-test-rs-zone-public.com.'
    zones = []

    def setUp(self):
        super(TestRecordsets, self).setUp()

        # create public zone
        try:
            self.zone = self.client.create_zone(
                name=self.zone_alias
            )
            self.client.wait_for_zone(self.zone)
        except openstack.exceptions.BadRequestException:
            self.zone = self.client.find_zone(self.zone_alias)
        self.zones.append(self.zone)

    def tearDown(self):
        try:
            for zone in self.zones:
                if zone:
                    self.client.delete_zone(zone)
                    self.client.wait_for_delete_zone(zone)
        except openstack.exceptions.SDKException as e:
            _logger.warning('Got exception during clearing resources %s'
                            % e.message)
        super(TestRecordsets, self).tearDown()

    def test_list_recordsets(self):
        rs = []
        recordsets = self.client.recordsets(self.zone.id)
        for record in recordsets:
            rs.append(record['records'])
        self.assertEqual(rs, [
            ['ns1.open-telekom-cloud.com. '
             'dl-otc-domains.telekom.de. (1 7200 900 1209600 300)'],
            ['ns2.open-telekom-cloud.com.', 'ns1.open-telekom-cloud.com.']])

    def test_create_recordset(self):
        rs = self.client.create_recordset(
            zone=self.zone.id,
            name=f'a-record.{self.zone_alias}',
            type='A',
            records=['1.1.1.1', '2.2.2.2']
        )
        self.client.wait_for_recordset(rs)
        self.assertEqual(rs.name, f'a-record.{self.zone_alias}')

        rs = self.client.create_recordset(
            zone=self.zone.id,
            name=f'aaaa-record.{self.zone_alias}',
            type='AAAA',
            records=['ff03:0db8:85a3:0:0:8a2e:0370:7334']
        )
        self.client.wait_for_recordset(rs)
        self.assertEqual(rs.name, f'aaaa-record.{self.zone_alias}')

        rs = self.client.create_recordset(
            zone=self.zone.id,
            name=f'cname-record.{self.zone_alias}',
            type='CNAME',
            records=['www.ex.com']
        )
        self.client.wait_for_recordset(rs)
        self.assertEqual(rs.name, f'cname-record.{self.zone_alias}')

        rs = self.client.create_recordset(
            zone=self.zone.id,
            name=f'mx-record.{self.zone_alias}',
            type='MX',
            records=['10 mailserver1.example.com.']
        )
        self.client.wait_for_recordset(rs)
        self.assertEqual(rs.name, f'mx-record.{self.zone_alias}')

        rs = self.client.create_recordset(
            zone=self.zone.id,
            name=f'txt-record.{self.zone_alias}',
            type='TXT',
            records=["\"Text.\""]
        )
        self.client.wait_for_recordset(rs)
        self.assertEqual(rs.name, f'txt-record.{self.zone_alias}')

        rs = self.client.create_recordset(
            zone=self.zone.id,
            name=f'ns-record.{self.zone_alias}',
            type='NS',
            records=['ns.example.com']
        )
        self.client.wait_for_recordset(rs)
        self.assertEqual(rs.name, f'ns-record.{self.zone_alias}')

    def test_get_recordset(self):
        rs = self.client.create_recordset(
            zone=self.zone.id,
            name=f'a-record.{self.zone_alias}',
            type='A',
            records=['1.1.1.1', '2.2.2.2']
        )
        self.client.wait_for_recordset(rs)
        record = self.client.get_recordset(
            recordset=rs.id,
            zone=self.zone.id
        )
        self.assertEqual(record.name, f'a-record.{self.zone_alias}')

    def test_find_recordset(self):
        rs = self.client.create_recordset(
            zone=self.zone.id,
            name=f'a-record.{self.zone_alias}',
            type='A',
            records=['1.1.1.1', '2.2.2.2']
        )
        self.client.wait_for_recordset(rs)
        record = self.client.find_recordset(
            name_or_id=rs.id,
            zone=self.zone.id
        )
        self.assertEqual(record.name, f'a-record.{self.zone_alias}')

    def test_update_recordset(self):
        rs = self.client.create_recordset(
            zone=self.zone.id,
            name=f'a-record.{self.zone_alias}',
            type='A',
            records=['1.1.1.1', '2.2.2.2']
        )
        self.client.wait_for_recordset(rs)
        record = self.client.update_recordset(
            recordset=rs.id,
            zone_id=self.zone.id,
            description='updated'
        )
        self.assertEqual(record.description, 'updated')

    def test_delete_recordset(self):
        rs = self.client.create_recordset(
            zone=self.zone.id,
            name=f'a-record.{self.zone_alias}',
            type='A',
            records=['1.1.1.1', '2.2.2.2']
        )
        self.client.wait_for_recordset(rs)
        self.assertEqual(rs.status, 'ACTIVE')
        record = self.client.delete_recordset(
            recordset=rs.id,
            zone=self.zone.id
        )
        self.client.wait_for_delete_recordset(record)
        self.assertIsNotNone(record)



#!/usr/bin/env python3
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.
"""
Import API
"""
import openstack
import json

openstack.enable_logging(True)
conn = openstack.connect(cloud='otc')
with open('/mnt/c/Users/sand1/PycharmProjects/python-otcextensions/'
          'otcextensions/tests/functional/sdk/apig/v2/test.json', 'r') as f:
    openapi_content = json.load(f)
attrs = {
    "is_create_group": False,
    "group_id": "ce973ff83ce54ef192c80bde884aa0ac",
    "file_name": "test.json",
    "swagger": openapi_content
}
conn.apig.client.import_api(
    gateway='gateway_id',
    **attrs)



#   Licensed under the Apache License, Version 2.0 (the "License"); you may
#   not use this file except in compliance with the License. You may obtain
#   a copy of the License at
#
#        http://www.apache.org/licenses/LICENSE-2.0
#
#   Unless required by applicable law or agreed to in writing, software
#   distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#   WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#   License for the specific language governing permissions and limitations
#   under the License.
#
'''DCS Statistics v1 action implementations'''
import logging

from osc_lib import utils
from osc_lib.command import command

from otcextensions.i18n import _

LOG = logging.getLogger(__name__)


class ListStatistic(command.Lister):
    _description = _('List DCS Statistics')
    columns = ('instance_id', 'max_memory', 'used_memory',
               'cmd_get_count', 'cmd_set_count', 'used_cpu',
               'input_kbps', 'output_kbps')

    def get_parser(self, prog_name):
        parser = super(ListStatistic, self).get_parser(prog_name)
        return parser

    def take_action(self, parsed_args):
        client = self.app.client_manager.dcs

        data = client.statistics()

        table = (self.columns,
                 (utils.get_item_properties(
                     s, self.columns,
                 ) for s in data))
        return table



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.
from keystoneauth1 import adapter

import mock

from openstack.tests.unit import base
from otcextensions.sdk.vpc.v1 import route


IDENTIFIER = 'ID'
EXAMPLE = {
    "type": "peering",
    "nexthop": "60c809cb-6731-45d0-ace8-3bf5626421a9",
    "destination": "192.168.200.0/24",
    "vpc_id": "ab78be2d-782f-42a5-aa72-35879f6890ff",
    "tenant_id": "6fbe9263116a4b68818cf1edce16bc4f",
    "id": "3d42a0d4-a980-4613-ae76-a2cddecff054"
}


class TestRoute(base.TestCase):

    def setUp(self):
        super(TestRoute, self).setUp()
        self.sess = mock.Mock(spec=adapter.Adapter)
        self.sess.put = mock.Mock()

    def test_basic(self):
        sot = route.Route()
        self.assertEqual('route', sot.resource_key)
        self.assertEqual('routes', sot.resources_key)
        path = '/v2.0/vpc/routes'
        self.assertEqual(path, sot.base_path)
        self.assertTrue(sot.allow_list)
        self.assertTrue(sot.allow_create)
        self.assertTrue(sot.allow_fetch)
        self.assertTrue(sot.allow_delete)
        self.assertFalse(sot.allow_commit)

    def test_make_it(self):
        sot = route.Route(**EXAMPLE)
        self.assertEqual(EXAMPLE['id'], sot.id)
        self.assertEqual(EXAMPLE['destination'], sot.destination)
        self.assertEqual(EXAMPLE['nexthop'], sot.nexthop)
        self.assertEqual(EXAMPLE['type'], sot.type)
        self.assertEqual(EXAMPLE['vpc_id'], sot.router_id)
        self.assertEqual(EXAMPLE['tenant_id'], sot.project_id)



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.
#
import mock
from openstackclient.tests.unit import utils
from osc_lib import utils as _osc_lib_utils

from otcextensions.sdk.modelartsv2.v2 import dataset
from otcextensions.sdk.modelartsv2.v2 import dataset_export_task
from otcextensions.sdk.modelartsv2.v2 import dataset_import_task
from otcextensions.sdk.modelartsv2.v2 import dataset_sample
from otcextensions.sdk.modelartsv2.v2 import dataset_statistics
from otcextensions.sdk.modelartsv2.v2 import dataset_sync
from otcextensions.sdk.modelartsv2.v2 import dataset_version
from otcextensions.tests.unit.osclient import test_base
from otcextensions.tests.unit.sdk.modelartsv2.v2 import examples


def gen_data(obj, columns, formatters=None):
    """Fill expected data tuple based on columns list"""
    return _osc_lib_utils.get_item_properties(
        obj, columns, formatters=formatters
    )


class TestModelartsv2(utils.TestCommand):
    def setUp(self):
        super(TestModelartsv2, self).setUp()
        self.app.client_manager.modelartsv2 = mock.Mock()
        self.client = self.app.client_manager.modelartsv2


class FakeDataset(test_base.Fake):
    """Fake one or more Modelarts dataset."""

    @classmethod
    def generate(cls):
        """Create a fake dataset sample.

        :return:
            A FakeResource object, with id, name and so on
        """
        return dataset.Dataset(**examples.DATASET)


class FakeDatasetVersion(test_base.Fake):
    """Fake one or more Modelarts dataset."""

    @classmethod
    def generate(cls):
        """Create a fake dataset sample.

        :return:
            A FakeResource object, with id, name and so on
        """
        return dataset_version.DatasetVersion(**examples.DATASET_VERSION)


class FakeDatasetStatistics(test_base.Fake):
    """Fake one or more Modelarts dataset Sample."""

    @classmethod
    def generate(cls):
        """Create a fake dataset statistics.

        :return:
            A FakeResource object, with id, name and so on
        """
        return dataset_statistics.DatasetStatistics(
            **examples.DATASET_STATISTICS
        )


class FakeDatasetSample(test_base.Fake):
    """Fake one or more Modelarts dataset Sample."""

    @classmethod
    def generate(cls):
        """Create a fake dataset sample.

        :return:
            A FakeResource object, with id, name and so on
        """
        return dataset_sample.DatasetSample(**examples.DATASET_SAMPLE)


class FakeDatasetSampleResp(test_base.Fake):
    """Fake dataset sample delete response."""

    @classmethod
    def generate(cls):
        object_info = {
            "success": False,
            "results": [
                {
                    "success": False,
                    "error_code": "ModelArts.4420",
                    "error_msg": "Sample not found. [sample-id]",
                },
                {
                    "success": True,
                },
            ],
        }
        return dataset_sample.DatasetSample(**object_info)


class FakeDatasetSync(test_base.Fake):
    """Fake dataset sample delete response."""

    @classmethod
    def generate(cls):
        object_info = {
            "status": "COMPLETED",
            "dataset_id": "gfghHSokody6AJigS5A",
        }

        return dataset_sync.DatasetSync(**object_info)


class FakeDatasetImportTask(test_base.Fake):
    """Fake dataset export task response."""

    @classmethod
    def generate(cls):
        return dataset_import_task.DatasetImportTask(
            **examples.DATASET_IMPORT_TASK
        )


class FakeDatasetExportTask(test_base.Fake):
    """Fake dataset export task response."""

    @classmethod
    def generate(cls):
        return dataset_export_task.DatasetExportTask(
            **examples.DATASET_EXPORT_TASK
        )



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.
from openstack import exceptions
from openstack import resource


class Resource(resource.Resource):

    @classmethod
    def find(cls, session, name_or_id, ignore_missing=True, **params):
        """Find a resource by its name or id.

        :param session: The session to use for making this request.
        :type session: :class:`~keystoneauth1.adapter.Adapter`
        :param name_or_id: This resource's identifier, if needed by
                           the request. The default is ``None``.
        :param bool ignore_missing: When set to ``False``
                    :class:`~openstack.exceptions.ResourceNotFound` will be
                    raised when the resource does not exist.
                    When set to ``True``, None will be returned when
                    attempting to find a nonexistent resource.
        :param dict params: Any additional parameters to be passed into
                            underlying methods, such as to
                            :meth:`~openstack.resource.Resource.existing`
                            in order to pass on URI parameters.

        :return: The :class:`Resource` object matching the given name or id
                 or None if nothing matches.
        :raises: :class:`openstack.exceptions.DuplicateResource` if more
                 than one resource is found for this request.
        :raises: :class:`openstack.exceptions.ResourceNotFound` if nothing
                 is found and ignore_missing is ``False``.
        """
        session = cls._get_session(session)
        # Try to short-circuit by looking directly for a matching ID.
        try:
            match = cls.existing(
                id=name_or_id,
                connection=session._get_connection(),
                **params)
            return match.fetch(session, **params)
        except exceptions.SDKException:
            pass

        data = cls.list(session, **params)

        result = cls._get_one_match(name_or_id, data)
        if result is not None:
            return result

        if ignore_missing:
            return None
        raise exceptions.ResourceNotFound(
            "No %s found for %s" % (cls.__name__, name_or_id))

    @classmethod
    def list_override(cls, session, **params):
        # Some really stupid APIs are not following any guidelines
        # and are under other endpoint (without project id),
        # so we need to hack on the endpoint_override yet again
        session = cls._get_session(session)

        base_path = cls.base_path
        params.pop('paginated', None)
        params.pop('base_path', None)
        params = cls._query_mapping._validate(
            params, base_path=base_path,
            allow_unknown_params=False)
        query_params = cls._query_mapping._transpose(params, cls)
        uri = base_path % params
        region_id = None

        # Copy query_params due to weird mock unittest interactions
        response = session.get(
            uri,
            endpoint_override=session.endpoint_override.replace(
                '%(project_id)s', ''),
            headers={"Accept": "application/json"},
            params=query_params.copy())
        exceptions.raise_from_response(response)
        data = response.json()
        if 'regionId' in data:
            region_id = data['regionId']

        if cls.resources_key:
            resources = data[cls.resources_key]
        else:
            resources = data

        if not isinstance(resources, list):
            resources = [resources]

        for raw_resource in resources:
            if region_id:
                raw_resource['region_id'] = region_id
            value = cls.existing(
                connection=session._get_connection(),
                **raw_resource)
            yield value



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

from openstack.tests.unit import base

from otcextensions.sdk.function_graph.v2 import quota

EXAMPLE = {
    "quotas": {
        "resources": [{
            "quota": 60,
            "used": 3,
            "type": "fgs_func_scale_down_timeout"
        }, {
            "quota": 100,
            "used": 22,
            "type": "fgs_func_occurs"
        }, {
            "quota": 100,
            "used": 22,
            "type": "fgs_func_pat_idle_time"
        }, {
            "quota": 100,
            "used": 22,
            "type": "fgs_func_num"
        }, {
            "quota": 10240,
            "used": 22,
            "type": "fgs_func_code_size",
            "unit": "MB"
        }, {
            "quota": 512,
            "used": 22,
            "type": "fgs_workflow_num"
        }]
    }
}


class TestFunctionQuota(base.TestCase):

    def test_basic(self):
        sot = quota.Quota()
        path = '/fgs/quotas'
        self.assertEqual(path, sot.base_path)
        self.assertTrue(sot.allow_list)

    def test_make_it(self):
        sot = quota.Quota(**EXAMPLE)
        self.assertEqual(EXAMPLE['quotas'], sot.quotas)



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.
from openstack import exceptions
from openstack import resource


class Resource(resource.Resource):

    query_marker_key = 'offset'

    _query_mapping = resource.QueryParameters(
        'marker', 'limit',
        marker='offset'
    )

    @classmethod
    def list(cls, session, paginated=True, base_path=None,
             allow_unknown_params=False, **params):

        if not cls.allow_list:
            raise exceptions.MethodNotSupported(cls, "list")
        session = cls._get_session(session)
        microversion = cls._get_microversion(session)

        if base_path is None:
            base_path = cls.base_path
        params = cls._query_mapping._validate(
            params, base_path=base_path,
            allow_unknown_params=allow_unknown_params)

        query_params = cls._query_mapping._transpose(params, cls)
        uri = base_path % params

        limit = query_params.get('limit')

        # Track the total number of resources yielded so we can paginate
        # swift objects
        total_yielded = query_params.get('offset', 0)
        while uri:
            # Copy query_params due to weird mock unittest interactions
            response = session.get(
                uri,
                headers={"Accept": "application/json"},
                params=query_params.copy(),
                microversion=microversion)
            exceptions.raise_from_response(response)
            data = response.json()

            # Discard any existing pagination keys
            query_params.pop('offset', None)
            query_params.pop('limit', None)

            if cls.resources_key:
                resources = data[cls.resources_key]
            else:
                resources = data

            if not isinstance(resources, list):
                resources = [resources]

            marker = None
            for raw_resource in resources:
                value = cls.existing(
                    microversion=microversion,
                    connection=session._get_connection(),
                    **raw_resource)
                marker = total_yielded + 1
                yield value
                total_yielded += 1

            if resources and paginated:
                uri, next_params = cls._get_next_link(
                    uri, response, data, marker, limit, total_yielded)
                query_params.update(next_params)
            else:
                return

    @classmethod
    def _get_next_link(cls, uri, response, data, marker, limit, total_yielded):
        # RDS service pagination. Returns query for the next page
        next_link = None
        params = {}
        if 'total_count' in data and total_yielded <= data['total_count']:
            next_link = uri
            params['offset'] = marker
            params['limit'] = limit
        else:
            next_link = None
        query_params = cls._query_mapping._transpose(params, cls)
        return next_link, query_params

    @classmethod
    def find(cls, session, name_or_id, ignore_missing=True, **params):
        """Find a resource by its name or id.

        :param session: The session to use for making this request.
        :type session: :class:`~keystoneauth1.adapter.Adapter`
        :param name_or_id: This resource's identifier, if needed by
                           the request. The default is ``None``.
        :param bool ignore_missing: When set to ``False``
                    :class:`~openstack.exceptions.ResourceNotFound` will be
                    raised when the resource does not exist.
                    When set to ``True``, None will be returned when
                    attempting to find a nonexistent resource.
        :param dict params: Any additional parameters to be passed into
                            underlying methods, such as to
                            :meth:`~openstack.resource.Resource.existing`
                            in order to pass on URI parameters.

        :return: The :class:`Resource` object matching the given name or id
                 or None if nothing matches.
        :raises: :class:`openstack.exceptions.DuplicateResource` if more
                 than one resource is found for this request.
        :raises: :class:`openstack.exceptions.ResourceNotFound` if nothing
                 is found and ignore_missing is ``False``.
        """
        session = cls._get_session(session)
        # Try to short-circuit by looking directly for a matching ID.
        try:
            match = cls.existing(
                id=name_or_id,
                connection=session._get_connection(),
                **params)
            return match.fetch(session, **params)
        except exceptions.SDKException:
            pass

        data = cls.list(session, **params)

        result = cls._get_one_match(name_or_id, data)
        if result is not None:
            return result

        if ignore_missing:
            return None
        raise exceptions.ResourceNotFound(
            "No %s found for %s" % (cls.__name__, name_or_id))

    def delete(self, session, error_message=None):

        response = self._raw_delete(session)
        kwargs = {}
        if error_message:
            kwargs['error_message'] = error_message

        self._translate_response(response, has_body=True, **kwargs)
        return self



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.
#

DATASET = {
    "dataset_id": "dataset-id",
    "dataset_name": "dataset-name",
    "dataset_type": 0,
    "data_format": "Default",
    "next_version_num": 2,
    "status": 1,
    "data_sources": [{"data_type": 0, "data_path": "/pdinput/"}],
    "create_time": 1694604098240,
    "update_time": 1694604098240,
    "description": "",
    "current_version_id": "curr-version-id",
    "current_version_name": "V001",
    "total_sample_count": 176244,
    "annotated_sample_count": 176244,
    "unconfirmed_sample_count": 0,
    "work_path": "/pdoutput/",
    "inner_work_path": "/",
    "inner_annotation_path": "/annotation/",
    "inner_data_path": "/data/",
    "inner_log_path": "/logs/",
    "inner_temp_path": "/temp/",
    "inner_task_path": "/task/",
    "work_path_type": 0,
    "workspace_id": "0",
    "enterprise_project_id": "0",
    "workforce_task_count": 0,
    "feature_supports": ["0"],
    "managed": False,
    "import_data": False,
    "ai_project": "default-ai-project",
    "label_task_count": 1,
    "dataset_format": 0,
    "dataset_version_count": 1,
    "dataset_version": "v1",
    "content_labeling": True,
    "data_update_time": 1704703808239,
    "versions": [
        {
            "version_id": "version-id",
            "version_name": "V001",
            "version_format": "Default",
            "status": 1,
            "is_current": True,
            "train_evaluate_sample_ratio": "0.80000",
            "export_images": False,
            "extract_serial_number": False,
        }
    ],
    "labels": [
        {
            "name": "Apple__Apple_scab",
            "type": 0,
            "property": {"@modelarts:color": "", "@modelarts:shortcut": ""},
            "attributes": [],
        },
        {
            "name": "Tomato_Septoria_leaf_spot",
            "type": 0,
            "property": {"@modelarts:color": "", "@modelarts:shortcut": ""},
            "attributes": [],
        },
        {
            "name": "tulips",
            "type": 0,
            "property": {"@modelarts:color": "", "@modelarts:shortcut": ""},
            "attributes": [],
        },
    ],
}


DATASET_EXPORT_TASK = {
    "task_id": "TZMuy7OKbClkGCAc3gb",
    "path": "/test-obs/daoChu/",
    "export_type": 3,
    "version_format": "Default",
    "export_format": 2,
    "export_params": {
        "sample_state": "",
        "export_dest": "DIR",
        "clear_hard_property": True,
        "clear_difficult": False,
        "train_sample_ratio": 1.0,
        "ratio_sample_usage": False,
    },
    "status": "RUNNING",
    "progress": 0.0,
    "create_time": 1606103424662,
    "update_time": 1606103494124,
}


DATASET_IMPORT_TASK = {
    "status": "COMPLETED",
    "task_id": "gfghHSokody6AJigS5A_RHJ1zOkIoI3Nzwxj8nh",
    "dataset_id": "gfghHSokody6AJigS5A",
    "import_path": "obs://test-obs/daoLu_images/cat-rabbit/",
    "import_type": 0,
    "total_sample_count": 20,
    "imported_sample_count": 20,
    "annotated_sample_count": 20,
    "total_sub_sample_count": 0,
    "imported_sub_sample_count": 0,
    "total_file_size": 0,
    "finished_file_count": 0,
    "finished_file_size": 0,
    "total_file_count": 0,
    "update_ms": 1606114833955,
    "create_time": 1606114833874,
    "elapsed_time": 2,
}


DATASET_LABEL = {
    "name": "Cat",
    "property": {
        "@modelarts:color": "#3399ff",
        "@modelarts:default_shape": "bndbox",
    },
}

DATASET_SAMPLE = {
    "annotated_by": "human/OTC-EU-DE-000000000010000XXXXXX/dummy",
    "labels": [
        {
            "name": "Tomato_healthy",
            "property": {},
            "type": 0,
        },
    ],
    "sample_type": 0,
    "source": "https://dummydummy/testdata",
    "preview": "https://dummydummy/test",
    "sample_id": "000500f237d4c078ca64f2fd99da9828",
    "sample_status": "MANUAL_ANNOTATION",
    "sample_time": 1694457754000,
    "metadata": {
        "@modelarts:import_origin": 0,
        "@modelarts:size": [256, 256, 3],
        "@modelarts:source_image_info": "https://dummydummy/test",
    },
}


DATASET_STATISTICS = {
    "label_stats": [
        {
            "name": "daisy",
            "type": 0,
            "property": {"@modelarts:color": "#266b5e"},
            "count": 0,
            "sample_count": 0,
        },
        {
            "name": "dandelion",
            "type": 0,
            "property": {"@modelarts:color": "#1a0135"},
            "count": 0,
            "sample_count": 0,
        },
    ],
    "sample_stats": {
        "un_annotation": 500,
        "all": 500,
        "total": 500,
        "deleted": 0,
        "manual_annotation": 0,
        "auto_annotation": 0,
        "lefted": 500,
    },
    "key_sample_stats": {
        "total": 500,
        "non_key_sample": 500,
        "key_sample": 0,
    },
    "deletion_stats": {},
    "metadata_stats": {},
    "data_spliting_enable": False,
}


DATASET_VERSION = {
    "annotated_sample_count": 600,
    "create_time": 1707050585432,
    "data_path": "/path..",
    "description": "",
    "export_images": False,
    "extract_serial_number": False,
    "is_current": True,
    "label_stats": [
        {
            "name": "Potato___Early_blight",
            "type": 0,
            "property": {"@modelarts:color": "", "@modelarts:shortcut": ""},
            "attributes": [],
            "count": 300,
            "sample_count": 300,
        },
        {
            "name": "Tomato___Bacterial_spot",
            "type": 0,
            "property": {"@modelarts:color": "", "@modelarts:shortcut": ""},
            "attributes": [],
            "count": 300,
            "sample_count": 300,
        },
    ],
    "label_type": "single",
    "manifest_path": "/path..",
    "previous_version_id": "previous-version-id",
    "remove_sample_usage": False,
    "status": 1,
    "task_type": 0,
    "total_sample_count": 1800,
    "train_evaluate_sample_ratio": "1.00",
    "version_format": "Default",
    "version_id": "version-id",
    "version_name": "V0fsfsfs04",
}



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.
#
"""VPC Route v2 action implementations"""
import logging

from osc_lib import utils
from osc_lib import exceptions
from osc_lib.command import command

from otcextensions.i18n import _

LOG = logging.getLogger(__name__)


def set_attributes_for_print(routes):
    for route in routes:
        yield route


def translate_response(func):
    def new(self, *args, **kwargs):
        response = func(self, *args, **kwargs)
        columns = (
            'id',
            'type',
            'nexthop',
            'destination',
            'router_id',
            'project_id'
        )
        data = utils.get_item_properties(response, columns)
        return (columns, data)
    new.__name__ = func.__name__
    new.__doc__ = func.__doc__
    return new


class ListVpcRoutes(command.Lister):

    _description = _("List Vpc Routes.")
    columns = (
        'Id',
        'Type',
        'Router Id',
        'Project Id',
        'NextHop',
        'Destination'
    )

    def get_parser(self, prog_name):
        parser = super(ListVpcRoutes, self).get_parser(prog_name)

        parser.add_argument(
            '--id',
            metavar='<id>',
            help=_("Specifies the ID of the VPC route."),
        )
        parser.add_argument(
            '--limit',
            metavar='<limit>',
            type=int,
            help=_("Limit to fetch number of records."),
        )
        parser.add_argument(
            '--marker',
            metavar='<marker>',
            help=_("Specifies the start resource ID of pagination query."),
        )
        parser.add_argument(
            '--project-id',
            metavar='<project_id>',
            help=_("Specifies the project ID."),
        )
        parser.add_argument(
            '--router-id',
            metavar='<router_id>',
            help=_("Specifies the router/vpc ID."),
        )
        parser.add_argument(
            '--destination',
            metavar='<destination>',
            help=_("Specifies that the route destination address (CIDR) "
                   "is used as the filtering condition."),
        )
        parser.add_argument(
            '--type',
            metavar='<type>',
            help=_("Specifies that the type is used as the filtering "
                   "condition. Currently, the value can only be peering."),
        )
        return parser

    def take_action(self, parsed_args):
        client = self.app.client_manager.vpc
        args_list = [
            'id',
            'type',
            'limit',
            'marker',
            'project_id',
            'router_id',
            'destination']
        attrs = {}
        for arg in args_list:
            val = getattr(parsed_args, arg)
            if val:
                attrs[arg] = val

        data = client.routes(**attrs)
        if data:
            data = set_attributes_for_print(data)

        return (self.columns, (utils.get_item_properties(s, self.columns)
                               for s in data))


class ShowVpcRoute(command.ShowOne):
    _description = _("Show VPC Route Details.")

    def get_parser(self, prog_name):
        parser = super(ShowVpcRoute, self).get_parser(prog_name)
        parser.add_argument(
            'route',
            metavar='<route>',
            help=_("Specifies the ID of the VPC route."),
        )
        return parser

    @translate_response
    def take_action(self, parsed_args):
        client = self.app.client_manager.vpc
        return client.get_route(parsed_args.route)


class AddVpcRoute(command.ShowOne):
    _description = _("Add Vpc Route.")

    def get_parser(self, prog_name):
        parser = super(AddVpcRoute, self).get_parser(prog_name)
        parser.add_argument(
            '--destination',
            metavar='<destination>',
            required=True,
            help=_("Specifies the destination address in the CIDR "
                   "notation format, for example, 192.168.200.0/24."),
        )
        parser.add_argument(
            '--nexthop',
            metavar='<nexthop>',
            required=True,
            help=_("Specifies the next hop. If the type is "
                   "peering, enter the VPC peering connection ID."),
        )
        parser.add_argument(
            '--type',
            metavar='<type>',
            default='peering',
            help=_("Specifies the route type. Currently, the value can "
                   "only be peering."),
        )
        parser.add_argument(
            '--router-id',
            metavar='<router_id>',
            required=True,
            help=_("Specifies the requesting router ID for creating a route."),
        )
        return parser

    @translate_response
    def take_action(self, parsed_args):
        client = self.app.client_manager.vpc
        args_list = [
            'type',
            'router_id',
            'destination',
            'nexthop']
        attrs = {}
        for arg in args_list:
            val = getattr(parsed_args, arg)
            if val:
                attrs[arg] = val

        return client.add_route(**attrs)


class DeleteVpcRoute(command.Command):

    _description = _("Deletes VPC Route.")

    def get_parser(self, prog_name):
        parser = super(DeleteVpcRoute, self).get_parser(prog_name)
        parser.add_argument(
            'route',
            metavar='<route>',
            nargs='+',
            help=_("VPC Routes(s) ID to delete"),
        )
        return parser

    def take_action(self, parsed_args):
        client = self.app.client_manager.vpc
        result = 0
        for route in parsed_args.route:
            try:
                obj = client.get_route(route)
                client.delete_route(obj.id)
            except Exception as e:
                result += 1
                LOG.error(_("Failed to delete VPC route with "
                          "ID '%(route)s': %(e)s"),
                          {'route': route, 'e': e})
        if result > 0:
            total = len(parsed_args.route)
            msg = (_("%(result)s of %(total)s VPC route(s) failed "
                   "to delete.") % {'result': result, 'total': total})
            raise exceptions.CommandError(msg)



#   Licensed under the Apache License, Version 2.0 (the "License"); you may
#   not use this file except in compliance with the License. You may obtain
#   a copy of the License at
#
#        http://www.apache.org/licenses/LICENSE-2.0
#
#   Unless required by applicable law or agreed to in writing, software
#   distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#   WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#   License for the specific language governing permissions and limitations
#   under the License.
#
"""OBS Container v1 action implementations"""
import logging

from otcextensions.i18n import _

from osc_lib import utils
from osc_lib.cli import parseractions
from osc_lib.command import command

from otcextensions.common import sdk_utils


LOG = logging.getLogger(__name__)

STORAGE_CLASSES = ['STANDARD', 'STANDARD_IA', 'GLACIER']


def _get_columns(item):
    column_map = {
        # 'tenant_id': 'project_id',
        # 'is_ha': 'ha',
        # 'is_distributed': 'distributed',
        # 'is_admin_state_up': 'admin_state_up',
    }
    return sdk_utils.get_osc_show_columns_for_sdk_resource(item, column_map)


class CreateContainer(command.ShowOne):
    _description = _("Create new container")

    def get_parser(self, prog_name):
        parser = super(CreateContainer, self).get_parser(prog_name)
        parser.add_argument(
            'container',
            metavar='<container-name>',
            help=_('New container name(s)'),
        )
        parser.add_argument(
            '--storage-class',
            metavar='{' + ','.join(STORAGE_CLASSES) + '}',
            type=lambda s: s.upper(),
            choices=STORAGE_CLASSES,
            help=_('Storage class'),
        )
        return parser

    def take_action(self, parsed_args):
        # raise NotImplementedError
        attrs = {
            'name': parsed_args.container
        }

        if parsed_args.storage_class:
            attrs['storage_class'] = parsed_args.storage_class

        data = self.app.client_manager.obs.create_container(**attrs)
        display_columns, columns = _get_columns(data)
        data = utils.get_item_properties(data, columns)
        #
        return (display_columns, data)


class DeleteContainer(command.Command):
    _description = _("Delete container")

    def get_parser(self, prog_name):
        parser = super(DeleteContainer, self).get_parser(prog_name)
        parser.add_argument(
            '--recursive', '-r',
            action='store_true',
            default=False,
            help=_('Recursively delete objects and container'),
        )
        parser.add_argument(
            'containers',
            metavar='<container>',
            nargs="+",
            help=_('Container(s) to delete'),
        )
        return parser

    def take_action(self, parsed_args):
        for container in parsed_args.containers:
            if parsed_args.recursive:
                # TODO(agoncharov) do a mass delete
                # through the boto3.delete_objects
                objs = self.app.client_manager.obs.objects(
                    container=container)
                for obj in objs:
                    self.app.client_manager.obs.delete_object(
                        container=container,
                        object=obj['name'],
                    )
            self.app.client_manager.obs.delete_container(
                container=container,
            )


class ListContainer(command.Lister):
    _description = _("List containers")

    def get_parser(self, prog_name):
        parser = super(ListContainer, self).get_parser(prog_name)
        # parser.add_argument(
        #     "--prefix",
        #     metavar="<prefix>",
        #     help=_("Filter list using <prefix>"),
        # )
        # parser.add_argument(
        #     "--marker",
        #     metavar="<marker>",
        #     help=_("Anchor for paging"),
        # )
        # parser.add_argument(
        #     "--end-marker",
        #     metavar="<end-marker>",
        #     help=_("End anchor for paging"),
        # )
        # parser.add_argument(
        #     "--limit",
        #     metavar="<num-containers>",
        #     type=int,
        #     help=_("Limit the number of containers returned"),
        # )
        # parser.add_argument(
        #     '--long',
        #     action='store_true',
        #     default=False,
        #     help=_('List additional fields in output'),
        # )
        # parser.add_argument(
        #     '--all',
        #     action='store_true',
        #     default=False,
        #     help=_('List all containers (default is 10000)'),
        # )
        return parser

    def take_action(self, parsed_args):

        # if parsed_args.long:
        #     columns = ('Name', 'Bytes', 'Count')
        # else:
        columns = ('name', 'creation_date')

        kwargs = {}
        # if parsed_args.prefix:
        #     kwargs['prefix'] = parsed_args.prefix
        # if parsed_args.marker:
        #     kwargs['marker'] = parsed_args.marker
        # if parsed_args.end_marker:
        #     kwargs['end_marker'] = parsed_args.end_marker
        # if parsed_args.limit:
        #     kwargs['limit'] = parsed_args.limit
        # if parsed_args.all:
        #     kwargs['full_listing'] = True

        data = self.app.client_manager.obs.containers(
            **kwargs
        )

        return (columns,
                (utils.get_item_properties(
                    s, columns,
                    formatters={},
                ) for s in data))


class SaveContainer(command.Command):
    _description = _("Save container contents locally")

    def get_parser(self, prog_name):
        parser = super(SaveContainer, self).get_parser(prog_name)
        parser.add_argument(
            'container',
            metavar='<container>',
            help=_('Container to save'),
        )
        return parser

    def take_action(self, parsed_args):
        raise NotImplementedError
        self.app.client_manager.obs.container_save(
            container=parsed_args.container,
        )


class SetContainer(command.Command):
    _description = _("Set container properties")

    def get_parser(self, prog_name):
        parser = super(SetContainer, self).get_parser(prog_name)
        parser.add_argument(
            'container',
            metavar='<container>',
            help=_('Container to modify'),
        )
        parser.add_argument(
            "--property",
            metavar="<key=value>",
            required=True,
            action=parseractions.KeyValueAction,
            help=_("Set a property on this container "
                   "(repeat option to set multiple properties)")
        )
        return parser

    def take_action(self, parsed_args):
        raise NotImplementedError

        self.app.client_manager.obs.container_set(
            parsed_args.container,
            properties=parsed_args.property,
        )


class ShowContainer(command.ShowOne):
    _description = _("Display container details")

    def get_parser(self, prog_name):
        parser = super(ShowContainer, self).get_parser(prog_name)
        parser.add_argument(
            'container',
            metavar='<container>',
            help=_('Container to display'),
        )
        return parser

    def take_action(self, parsed_args):

        data = self.app.client_manager.obs.get_container(
            container=parsed_args.container,
        )

        display_columns, columns = _get_columns(data)
        data = utils.get_item_properties(data, columns)
        #
        return (display_columns, data)


class UnsetContainer(command.Command):
    _description = _("Unset container properties")

    def get_parser(self, prog_name):
        parser = super(UnsetContainer, self).get_parser(prog_name)
        parser.add_argument(
            'container',
            metavar='<container>',
            help=_('Container to modify'),
        )
        parser.add_argument(
            '--property',
            metavar='<key>',
            required=True,
            action='append',
            default=[],
            help=_('Property to remove from container '
                   '(repeat option to remove multiple properties)'),
        )
        return parser

    def take_action(self, parsed_args):
        raise NotImplementedError
        self.app.client_manager.obs.container_unset(
            parsed_args.container,
            properties=parsed_args.property,
        )



#!/usr/bin/env python3
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.
#
"""Modify a Dataset."""
import openstack

openstack.enable_logging(True)
conn = openstack.connect(cloud="otc")

dataset_id = "qOI24kz47Jpx6IdGGx8"
attrs = {
    "dataset_id": "gfghHSokody6AJigS5A",
    "description": "just a test",
    "add_tags": [
        {
            "name": "Bee",
            "type": 0,
            "property": {"@modelarts:color": "#3399ff"},
        }
    ],
}
conn.modelartsv2.update_dataset(dataset_id, **attrs)



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.
from openstack import resource

from otcextensions.sdk.cce.v3 import _base


class VolumeSpec(resource.Resource):
    # Properties
    #: Disk Size in GB.
    size = resource.Body('size', type=int)
    #: Volume type: [SATA, SAS, SSD].
    type = resource.Body('volumetype')


class StatusSpec(_base.StatusSpec):
    # Properties
    #: ID of the VM where the node resides in the ECS.
    instance_id = resource.Body('serverId')
    #: Elastic IP address of a node.
    floating_ip = resource.Body('publicIP')
    #: Private IP address of a node.
    private_ip = resource.Body('privateIP')


class PublicIPSpec(resource.Resource):
    # Properties:
    #: List of IDs for the existing floating ips.
    ids = resource.Body('ids', type=list)
    #: Count of the IP addresses to be dynamically created.
    count = resource.Body('count', type=int)
    #: Elastic IP address. Dict of {
    #:     type,
    #:     bandwidth:{
    #:        chargemode, size, sharetype
    #:     }
    #: }.
    floating_ip = resource.Body('eip', type=dict)


class TagSpec(resource.Resource):
    # Properties:
    #: Key of the tag
    key = resource.Body('key')
    #: Value of the tag
    value = resource.Body('value')


class TaintSpec(resource.Resource):
    # Properties:
    #: A key can contain letters, digits, hyphens, underscores and periods
    #: up to 63 characters starting with letter or digit.
    key = resource.Body('key')
    #: A value can contain letters, digits, hyphens, underscores and periods
    #: up to 63 characters starting with letter or digit.
    value = resource.Body('value')
    #: Effect available options: NoSchedule, PreferNoSchedule, NoExecute
    effect = resource.Body('effect')


class ExtendParamSpec(resource.Resource):
    # Properties:
    #: Script required before the installation
    #: Input must be Base64 encoded
    preinstall_script = resource.Body('alpha.cce/preInstall')
    #: Script required after the installation
    #: Input must be Base64 encoded
    postinstall_script = resource.Body('alpha.cce/postInstall')
    #: ConfigMap of the Docker data disk
    lvm_config = resource.Body('DockerLVMConfigOverride')
    #: Maximum number of pods on the node
    max_pods = resource.Body('maxPods', type=int)
    #: ID of a custom node image
    #: Mandatory if custom image is used on a bare metall node
    node_image_id = resource.Body('alpha.cce/NodeImageID')


class NodeSpec(resource.Resource):
    # Properties
    #: Name of the AZ where the node resides.
    availability_zone = resource.Body('az')
    #: Billing mode of a node. Currently, only pay-per-use is supported.
    billing_mode = resource.Body('billingMode', type=int, default=0)
    #: Number of nodes.
    count = resource.Body('count', type=int)
    #: Data disk parameters of a node. At present, only one data
    #: disk can be configured
    data_volumes = resource.Body('dataVolumes', type=list,
                                 list_type=VolumeSpec)
    #: ID of the Dedicated Host to which nodes will be scheduled
    dedicated_host = resource.Body('dedicatedHostId')
    #: ID of the ECS group where the CCE node can belong to
    ecs_group = resource.Body('ecsGroupId')
    # Extended parameters in key-value format
    extend_params = resource.Body('extendParam', type=ExtendParamSpec)
    #: The node is created in the specified fault domain.
    fault_domain = resource.Body('faultDomain')
    #: Flavor (mandatory)
    flavor = resource.Body('flavor')
    #: Elastic IP address parameters of a node.
    floating_ip = resource.Body('publicIP', type=PublicIPSpec)
    #: Kubernetes tags
    k8s_tags = resource.Body('k8sTags', type=dict)
    #: Parameters for logging in to the node.
    login = resource.Body('login')
    #: Boolean: if node is offloading all its components
    offload_node = resource.Body('offloadNode', type=bool)
    #: Operating System of the node. EulerOS and CentOS are supported.
    os = resource.Body('os')
    #: System disk parameters of the node.
    root_volume = resource.Body('rootVolume', type=VolumeSpec)
    #: Tags of a Node
    tags = resource.Body('userTags', type=list, list_type=TagSpec)
    #: Taints are used to configure anti-affinity
    taints = resource.Body('taints', type=list, list_type=TaintSpec)


class ClusterNode(_base.Resource):

    base_path = '/clusters/%(cluster_id)s/nodes'

    allow_create = True
    allow_fetch = True
    allow_commit = True
    allow_delete = True
    allow_list = True

    # Properties
    #: Cluster id.
    cluster_id = resource.URI('cluster_id')
    #: Spec
    spec = resource.Body('spec', type=NodeSpec)
    #: Status
    status = resource.Body('status', type=StatusSpec)

    @classmethod
    def new(cls, **kwargs):
        if 'kind' not in kwargs:
            kwargs['kind'] = 'Node'
        if 'apiVersion' not in kwargs:
            kwargs['apiVersion'] = 'v3'
        return cls(_synchronized=False, **kwargs)



#   Licensed under the Apache License, Version 2.0 (the "License"); you may
#   not use this file except in compliance with the License. You may obtain
#   a copy of the License at
#
#        http://www.apache.org/licenses/LICENSE-2.0
#
#   Unless required by applicable law or agreed to in writing, software
#   distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#   WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#   License for the specific language governing permissions and limitations
#   under the License.
import mock
import random

from otcextensions.osclient.dcs.v1 import config
from otcextensions.tests.unit.osclient.dcs.v1 import fakes


class TestListParam(fakes.TestDCS):

    objects = fakes.FakeConfig.create_multiple(3)
    inst = fakes.FakeInstance.create_one()

    columns = ('id', 'name', 'value', 'default_value')

    data = []

    for s in objects:
        data.append((
            s.id,
            s.name,
            s.value,
            s.default_value
        ))

    def setUp(self):
        super(TestListParam, self).setUp()

        self.cmd = config.ListInstanceParam(self.app, None)

        self.client.instance_params = mock.Mock()
        self.client.find_instance = mock.Mock()

    def test_list(self):
        arglist = [
            'inst'
        ]

        verifylist = [
            ('instance', 'inst')
        ]

        # Verify cm is triggereg with default parameters
        parsed_args = self.check_parser(self.cmd, arglist, verifylist)

        # Set the response
        self.client.instance_params.side_effect = [
            self.objects
        ]
        self.client.find_instance.side_effect = [
            self.inst
        ]

        # Trigger the action
        columns, data = self.cmd.take_action(parsed_args)

        self.client.instance_params.assert_called_once_with(
            instance={'id': self.inst.id},
        )

        self.assertEqual(self.columns, columns)
        self.assertEqual(self.data, list(data))


class TestShowParam(fakes.TestDCS):

    objects = fakes.FakeConfig.create_multiple(3)
    inst = fakes.FakeInstance.create_one()

    columns = (
        'default_value', 'description', 'id', 'name',
        'value', 'value_range', 'value_type')

    search = objects[random.randint(0, 2)]

    data = (
        search.default_value,
        search.description,
        search.id,
        search.name,
        search.value,
        search.value_range,
        search.value_type,
    )

    def setUp(self):
        super(TestShowParam, self).setUp()

        self.cmd = config.ShowInstanceParam(self.app, None)

        self.client.instance_params = mock.Mock()
        self.client.find_instance = mock.Mock()

    def test_list(self):
        criteria = self.search.name

        arglist = [
            'inst',
            '--param', criteria
        ]

        verifylist = [
            ('instance', 'inst'),
            ('param', criteria)
        ]

        # Verify cm is triggereg with default parameters
        parsed_args = self.check_parser(self.cmd, arglist, verifylist)

        # Set the response
        self.client.instance_params.side_effect = [
            self.objects
        ]
        self.client.find_instance.side_effect = [
            self.inst
        ]

        # Trigger the action
        columns, data = self.cmd.take_action(parsed_args)

        self.client.instance_params.assert_called_once_with(
            instance={'id': self.inst.id},
        )

        self.assertEqual(self.columns, columns)
        self.assertEqual(self.data, data)


class TestUpdateParam(fakes.TestDCS):

    inst = fakes.FakeInstance.create_one()

    def setUp(self):
        super(TestUpdateParam, self).setUp()

        self.cmd = config.UpdateInstanceParam(self.app, None)

        self.client.update_instance_params = mock.Mock()
        self.client.find_instance = mock.Mock()

    def test_update(self):

        arglist = [
            'inst',
            '--param', 'id1:name1:value1',
            '--param', 'id2:name2:value2',
        ]

        verifylist = [
            ('instance', 'inst'),
            ('param', ['id1:name1:value1', 'id2:name2:value2'])
        ]

        # Verify cm is triggereg with default parameters
        parsed_args = self.check_parser(self.cmd, arglist, verifylist)

        # Set the response
        self.client.update_instance_params.side_effect = [{}]
        self.client.find_instance.side_effect = [
            self.inst
        ]

        # Trigger the action
        self.cmd.take_action(parsed_args)

        self.client.update_instance_params.assert_called_once_with(
            instance={'id': self.inst.id},
            params=[
                {
                    'param_id': 'id1',
                    'param_name': 'name1',
                    'param_value': 'value1'
                }, {
                    'param_id': 'id2',
                    'param_name': 'name2',
                    'param_value': 'value2'
                }
            ]
        )



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.
from openstack import resource

from otcextensions.sdk.waf.v1 import _base


class Certificate(_base.Resource):
    """WAF Certificate Resource"""
    resources_key = 'items'
    base_path = '/waf/certificate'

    # capabilities
    allow_create = True
    allow_list = True
    allow_fetch = True
    allow_delete = True
    allow_commit = True

    _query_mapping = resource.QueryParameters(
        'limit', 'offset')

    #: Properties
    #: Specifies the certificate content.
    content = resource.Body('content')
    #: Certificate expiration time
    expire_time = resource.Body('expireTime')
    #: Specifies the private key content.
    key = resource.Body('key')
    #: Certificat uploading timestamp
    timestamp = resource.Body('timestamp')



#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

import json
import filecmp
import os
import uuid

import fixtures

from openstackclient.tests.functional import base


CONTAINER_CREATE_COMMAND = 'obs container create %(name)s -f json'
OBJECT_CREATE_COMMAND = 'obs object create %(container)s %(name)s -f json'

CONTAINER_DELETE_COMMAND = 'obs container delete %(name)s'
OBJECT_DELETE_COMMAND = 'obs object delete %(container)s %(name)s'


class ObsObjectTests(base.TestCase):
    """Functional tests for vbs. """

    CONTAINER_NAME = uuid.uuid4().hex
    OBJECT_NAME = uuid.uuid4().hex + '.tmp'

    OBJECT_CONTENT = uuid.uuid4().hex

    def tearDown(self):
        try:
            self.openstack(
                OBJECT_DELETE_COMMAND % {
                    'name': self.OBJECT_NAME,
                    'container': self.CONTAINER_NAME
                }
            )
            self.openstack(
                CONTAINER_DELETE_COMMAND % {'name': self.CONTAINER_NAME}
            )
            os.remove(self.OBJECT_NAME)
        finally:
            super(ObsObjectTests, self).tearDown()

    def setUp(self):
        super(ObsObjectTests, self).setUp()
        json_output = json.loads(self.openstack(
            CONTAINER_CREATE_COMMAND % {'name': self.CONTAINER_NAME}
        ))
        self.container_id = json_output["id"]
        self.assertOutput(self.CONTAINER_NAME, json_output['name'])
        with open(self.OBJECT_NAME, 'w') as file:
            file.write(self.OBJECT_CONTENT)
        json_output = json.loads(self.openstack(
            OBJECT_CREATE_COMMAND % {
                'name': self.OBJECT_NAME,
                'container': self.CONTAINER_NAME
            }
        ))
        self.object_id = json_output["id"]
        self.assertOutput(self.OBJECT_NAME, json_output['name'])

        ver_fixture = fixtures.EnvironmentVariable(
            'OS_OBS_API_VERSION', '1'
        )
        self.useFixture(ver_fixture)

    def test_object_list(self):
        json_output = json.loads(self.openstack(
            'obs object list %(container)s -f json ' % {
                'container': self.CONTAINER_NAME
            }
        ))
        self.assertIn(
            self.OBJECT_NAME,
            [cont['name'] for cont in json_output]
        )

    def test_object_save(self):
        download_name = self.OBJECT_NAME + '_output'
        self.openstack(
            'obs object save %(container)s %(name)s --file %(output)s' % {
                'container': self.CONTAINER_NAME,
                'name': self.OBJECT_NAME,
                'output': download_name
            }
        )
        self.assertTrue(filecmp.cmp(self.OBJECT_NAME, download_name))
        os.remove(download_name)



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.
from openstack import resource


class ApiQuantities(resource.Resource):
    base_path = f'/apigw/instances/%(gateway_id)s/resources/outline/apis'

    allow_fetch = True

    # Properties
    gateway_id = resource.URI('gateway_id')

    # Attributes
    # Total number of APIs.
    apis_num = resource.Body('instance_num', type=int)
    # Number of APIs that have been published in the RELEASE environment.
    apis_in_release = resource.Body('nums_on_release', type=int)
    # Number of APIs that have not been published in the RELEASE environment.
    apis_not_in_release = resource.Body('nums_off_release', type=int)


class ApiGroupQuantities(resource.Resource):
    base_path = f'/apigw/instances/%(gateway_id)s/resources/outline/groups'

    allow_fetch = True

    # Properties
    gateway_id = resource.URI('gateway_id')

    # Attributes
    # Number of API groups that have not been listed on KooGallery.
    not_in_koogallery = resource.Body('offsell_nums', type=int)
    # Number of API groups that have been listed on KooGallery.
    in_koogallery = resource.Body('onsell_nums', type=int)


class AppQuantities(resource.Resource):
    base_path = f'/apigw/instances/%(gateway_id)s/resources/outline/apps'

    allow_fetch = True

    # Properties
    gateway_id = resource.URI('gateway_id')

    # Attributes
    # Number of apps that have been authorized to access APIs.
    authorized = resource.Body('authed_nums', type=int)
    # Number of apps that have not been authorized to access APIs.
    not_authorized = resource.Body('unauthed_nums', type=int)



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.
from openstack.tests.unit import base
from otcextensions.sdk.apig.v2 import domain_name

EXAMPLE = {
    "url_domain": "www.company.com",
    "id": "c5e0d5ba62a34d26ad5c709ae22c1a17",
    "status": 3,
    "min_ssl_version": "TLSv1.1",
    "is_http_redirect_to_https": False,
    "verified_client_certificate_enabled": False
}


class TestDomainName(base.TestCase):

    def test_basic(self):
        sot = domain_name.DomainName()
        self.assertEqual(
            '/apigw/instances/%(gateway_id)s/'
            'api-groups/%(group_id)s/domains',
            sot.base_path)
        self.assertTrue(sot.allow_create)
        self.assertTrue(sot.allow_delete)
        self.assertTrue(sot.allow_fetch)

    def test_make_it(self):
        sot = domain_name.DomainName(**EXAMPLE)
        self.assertEqual(EXAMPLE['url_domain'], sot.url_domain)
        self.assertEqual(EXAMPLE['id'], sot.id)
        self.assertEqual(EXAMPLE['status'], sot.status)
        self.assertEqual(EXAMPLE['min_ssl_version'], sot.min_ssl_version)


EXAMPLE_CERTIFICATE = {
    "ssl_name": "cert_demo",
    "url_domain": "www.example.com",
    "ssl_id": "a27be832f2e9441c8127fe48e3b5ac67",
    "id": " f6bb84ccf1c34035878aa51b7253b21c",
    "status": 3
}


class TestCertificate(base.TestCase):

    def test_basic(self):
        sot = domain_name.Certificate()
        self.assertEqual(
            '/apigw/instances/%(gateway_id)s/'
            'api-groups/%(group_id)s/domains/'
            '%(domain_id)s/certificate',
            sot.base_path)
        self.assertTrue(sot.allow_create)
        self.assertTrue(sot.allow_delete)
        self.assertTrue(sot.allow_fetch)

    def test_make_it(self):
        sot = domain_name.Certificate(**EXAMPLE_CERTIFICATE)
        self.assertEqual(EXAMPLE_CERTIFICATE['ssl_name'], sot.ssl_name)
        self.assertEqual(EXAMPLE_CERTIFICATE['url_domain'], sot.url_domain)
        self.assertEqual(EXAMPLE_CERTIFICATE['ssl_id'], sot.ssl_id)
        self.assertEqual(EXAMPLE_CERTIFICATE['id'], sot.id)
        self.assertEqual(EXAMPLE_CERTIFICATE['status'], sot.status)


EXAMPLE_DEBUG = {
    "sl_domain_access_enabled": True
}


class TestDomainDebug(base.TestCase):

    def test_basic(self):
        sot = domain_name.DomainDebug()
        self.assertEqual(
            '/apigw/instances/%(gateway_id)s/api-groups/'
            '%(group_id)s/sl-domain-access-settings',
            sot.base_path)
        self.assertTrue(sot.allow_commit)

    def test_make_it(self):
        sot = domain_name.DomainDebug(**EXAMPLE_DEBUG)
        self.assertEqual(
            EXAMPLE_DEBUG['sl_domain_access_enabled'],
            sot.sl_domain_access_enabled
        )
